//
//	nsl_matrix.h
//
// 96-1-26 aa: add nsl_model
class nsl_model;

struct nsl_matrix : public nsl_layer
{
	num_type** v;
	int i0,i1,j0,j1;
	int imax,jmax;
public:
		nsl_matrix();
		nsl_matrix(int,int);
		nsl_matrix(nsl_matrix&);
// add layer to nsl_model
		nsl_matrix(nsl_model*,char*,int,int);
		nsl_matrix(nsl_model&,char*,int,int);
		nsl_matrix(char*,int,int); // NSL_MODEL
		~nsl_matrix();

	// alloc memory
	int	alloc_mem(int,int);

	// old get
	int	get_x0() { return i0; }
	int	get_x1() { return i1; }
	int	get_y0() { return j0; }
	int	get_y1() { return j1; }
	int	get_xn() { return imax; }
	int	get_yn() { return jmax; }
	// get
	int	get_i0() { return i0; }
	int	get_i1() { return i1; }
	int	get_j0() { return j0; }
	int	get_j1() { return j1; }
	int	get_imax() { return imax; }
	int	get_jmax() { return jmax; }
	int	get_rows() { return imax; }
	int	get_cols() { return jmax; }

	num_type**	get_matrix() { return v; }

// element retrieval
num_type&	elem(int,int);

// 	assignment
void 	operator=(nsl_matrix&);
void 	operator=(nsl_data&);
// void 	operator=(num_type**);
void 	operator=(num_type);

// 	arithmetic

// addition
friend	nsl_matrix 	operator+(nsl_matrix&);
friend	nsl_matrix 	operator+(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	operator+(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	operator+(nsl_data&,nsl_matrix&);
friend	nsl_matrix 	operator+(nsl_matrix&,num_type);
friend	nsl_matrix 	operator+(num_type,nsl_matrix&);

// subtraction
friend	nsl_matrix 	operator-(nsl_matrix&);
friend	nsl_matrix 	operator-(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	operator-(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	operator-(nsl_data&,nsl_matrix&);
friend	nsl_matrix 	operator-(nsl_matrix&,num_type);
friend	nsl_matrix 	operator-(num_type,nsl_matrix&);

// pointwise multiplication
friend	nsl_matrix 	operator^(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	operator^(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	operator^(nsl_data&,nsl_matrix&);
friend	nsl_matrix 	operator^(nsl_matrix&,num_type);
friend	nsl_matrix 	operator^(num_type,nsl_matrix&);

// pointwise division
friend	nsl_matrix 	operator/(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	operator/(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	operator/(nsl_data&,nsl_matrix&);
friend	nsl_matrix 	operator/(nsl_matrix&,num_type);
friend	nsl_matrix 	operator/(num_type,nsl_matrix&);

// convolution
friend	nsl_matrix 	operator*(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	operator*(nsl_vector&,nsl_matrix&);
friend	nsl_matrix 	operator*(nsl_matrix&,nsl_vector&);
friend	nsl_matrix 	operator*(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	operator*(nsl_data&,nsl_matrix&);
friend	nsl_matrix 	operator*(nsl_matrix&,num_type);
friend	nsl_matrix 	operator*(num_type,nsl_matrix&);

//	threshold functions

// NSLstep
friend	nsl_matrix 	NSLstep(nsl_matrix&);
friend	nsl_matrix 	NSLstep(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLstep(nsl_matrix&,num_type);
friend	nsl_matrix 	NSLstep(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	NSLstep(nsl_matrix&,nsl_matrix&,nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLstep(nsl_matrix&,nsl_matrix&,num_type,num_type);
friend	nsl_matrix 	NSLstep(nsl_matrix&,nsl_matrix&,nsl_data&,nsl_data&);
friend	nsl_matrix 	NSLstep(nsl_matrix&,num_type,num_type,num_type);
friend	nsl_matrix 	NSLstep(nsl_matrix&,nsl_data&,nsl_data&,nsl_data&);

// NSLramp
friend	nsl_matrix 	NSLramp(nsl_matrix&);
friend	nsl_matrix 	NSLramp(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLramp(nsl_matrix&,num_type);
friend	nsl_matrix 	NSLramp(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	NSLramp(nsl_matrix&,nsl_matrix&,nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLramp(nsl_matrix&,nsl_matrix&,num_type,num_type);
friend	nsl_matrix 	NSLramp(nsl_matrix&,nsl_matrix&,nsl_data&,nsl_data&);
friend	nsl_matrix 	NSLramp(nsl_matrix&,num_type,num_type,num_type);
friend	nsl_matrix 	NSLramp(nsl_matrix&,nsl_data&,nsl_data&,nsl_data&);

// NSLsat
friend	nsl_matrix 	NSLsat(nsl_matrix&);
friend	nsl_matrix 	NSLsat(nsl_matrix&,nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLsat(nsl_matrix&,num_type,num_type);
friend	nsl_matrix 	NSLsat(nsl_matrix&,nsl_data&,nsl_data&);
friend	nsl_matrix 	NSLsat(nsl_matrix&,nsl_matrix&,nsl_matrix&,
				nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLsat(nsl_matrix&,nsl_matrix&,nsl_matrix&,
				num_type,num_type);
friend	nsl_matrix 	NSLsat(nsl_matrix&,nsl_matrix&,nsl_matrix&,
				nsl_data&,nsl_data&);
friend	nsl_matrix 	NSLsat(nsl_matrix&,num_type,num_type,
				num_type,num_type);
friend	nsl_matrix 	NSLsat(nsl_matrix&,nsl_data&,nsl_data&,
				nsl_data&,nsl_data&);

// NSLsigma
friend	nsl_matrix 	NSLsigma(nsl_matrix&);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,num_type,num_type);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,nsl_data&,nsl_data&);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,nsl_matrix&,nsl_matrix&,
				nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,nsl_matrix&,nsl_matrix&,
				num_type,num_type);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,nsl_matrix&,nsl_matrix&,
				nsl_data&,nsl_data&);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,num_type,num_type,
				num_type,num_type);
friend	nsl_matrix 	NSLsigma(nsl_matrix&,nsl_data&,nsl_data&,
				nsl_data&,nsl_data&);

// 	max,min functions

// NSLmax
	num_type	max();
friend	num_type 	NSLmax(nsl_matrix&);
friend	nsl_matrix 	NSLmax(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLmax(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	NSLmax(nsl_matrix&,num_type);

// NSLmin
	num_type	min();
friend	num_type 	NSLmin(nsl_matrix&);
friend	nsl_matrix 	NSLmin(nsl_matrix&,nsl_matrix&);
friend	nsl_matrix 	NSLmin(nsl_matrix&,nsl_data&);
friend	nsl_matrix 	NSLmin(nsl_matrix&,num_type);

// misc
friend	nsl_vector 	NSLmat_to_vec(nsl_vector&,nsl_matrix&);
friend	nsl_vector 	NSLmat_to_col_vec(nsl_vector&,nsl_matrix&);
friend	nsl_vector 	NSLmat_to_row_vec(nsl_vector&,nsl_matrix&);
friend	nsl_matrix 	NSLvec_to_mat(nsl_matrix&,nsl_vector&);
friend	nsl_matrix 	NSLcol_vec_to_mat(nsl_matrix&,nsl_vector&);
friend	nsl_matrix 	NSLrow_vec_to_mat(nsl_matrix&,nsl_vector&);

	nsl_vector reduce_row();
	nsl_vector reduce_col();

	// sector
	nsl_matrix 	get_sector(int,int,int,int);
	nsl_matrix 	set_sector(nsl_matrix&,int,int);
	nsl_matrix 	put_sector(nsl_matrix&,int,int); // same as set_sector

	nsl_vector	get_row(int);
	nsl_vector	get_col(int);
	nsl_matrix	set_row(nsl_vector&,int);
	nsl_matrix	set_col(nsl_vector&,int);

	// sum
	num_type	sum();

	// print
friend	ostream& 	operator<<(ostream&,nsl_matrix&);
friend	istream& 	operator>>(istream&,nsl_matrix&);

	void	print(ostream&);
	void	print(ostream&,int,int,int,int);
// print status
	virtual void	print_status(ostream&); // virtual
		void	print_layer_status(ostream&); // local

	virtual void	print_stim(ostream&) {}
	virtual void	print_stim(ostream&,int) {}

	virtual nsl_stim_obj* get_stim(int) { return (nsl_stim_obj*) 0; }
	virtual nsl_stim_obj* get_stim(char*) { return (nsl_stim_obj*) 0; }
};

typedef nsl_matrix NSL_MATRIX;
typedef nsl_matrix NSL_MATRIX_LAYER;
