//
//	area_level_graph.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern void draw_area_level_matrix(Canvas,num_type**,num_type**,
	int,int,int,int,int,int,int,int,num_type,num_type,int);
extern void draw_area_level_vector(Canvas,num_type*,num_type*,
	int,int,int,int,int,int,num_type,num_type,int,int);
extern void draw_area_level_element(Canvas,num_type,num_type,
	int,int,int,int,num_type,num_type,int);

extern int draw_area_level_val(Canvas,int,int,int,int,float,float,float);

extern void set_all_canvas_events(Canvas);

area_level_graph::area_level_graph(display_window* window)
	: display_graph(window,"area_level_graph") 
{
	set_all_canvas_events(canvas);
	parent_canvas->set_clear_fg(0);
}
void area_level_graph::read_from_database()
{
	display_graph::read_from_database();
}
void area_level_graph::write_to_database()
{
	display_graph::write_to_database();
}
void area_level_graph::input(int x_pos,int y_pos,int fg)
{
}
void area_level_graph::output(int x_pos,int y_pos)
{
}
void area_level_graph::display()
{
}
void area_level_graph::print(ostream& out)
{
	canvas_object::print(out);
}

//par 
area_level_graph_par::area_level_graph_par(display_window* window,
	nsl_data* m) : area_level_graph(window)
{
	layer_ptr = m;
	layer0_ptr = new nsl_data;
}
void area_level_graph_par::read_from_database()
{
	parent_window->set_win_sector(0,0,0,0,1,1); // update win sector
	area_level_graph::read_from_database(); // read in new win sector
}
void area_level_graph_par::write_to_database()
{
	area_level_graph::write_to_database();
}
void area_level_graph_par::input(int x_pos,int y_pos,int fg)
{
	float val;

	num_type m0 = ((nsl_data*) layer0_ptr)->get_data();
	num_type m = ((nsl_data*) layer_ptr)->get_data();

	if (fg == 1)
	   val = 0.0;
	else
	   parent_window->get_general_data("val_in",&val);

	if (y_pos <= wy1 && y_pos >= wy0 && x_pos <= wx1 && x_pos >= wx0)
	{
    	    draw_area_level_val(canvas,wx0,wy0,wxs,wys,val,wymin,wymax);

	    ((nsl_data*) layer0_ptr)->set_value(val);
	    ((nsl_data*) layer_ptr)->set_value(val);
	}
}
void area_level_graph_par::output(int x_pos,int y_pos)
{
	float val;

	if (y_pos <= wy1 && y_pos >= wy0 && x_pos <= wx1 && x_pos >= wx0)
	{
	    val = ((nsl_data*) layer_ptr)->get_value();

	    parent_window->set_general_data("val_out",val);
	}
}
void area_level_graph_par::display()
{
	int fg; // redraw fg

	num_type m0 = ((nsl_data*) layer0_ptr)->get_data();
	num_type m = ((nsl_data*) layer_ptr)->get_data();

	if ((fg = parent_canvas->get_redraw_fg()) == 1)
		parent_canvas->set_redraw_fg(0);

	draw_area_level_element(canvas,m,m0,wx0,wy0,wxs,wys,wymin,wymax,fg);

	((nsl_data*) layer0_ptr)->set_value(m);
}
void area_level_graph_par::print(ostream& out)
{
	area_level_graph::print(out);
}

// vec 
area_level_graph_vec::area_level_graph_vec(display_window* window,
	nsl_vector* m) : area_level_graph(window)
{
	layer_ptr = m;

	imax = ((nsl_vector*) layer_ptr)->get_imax();
	
	layer0_ptr = new nsl_vector(imax);
}
void area_level_graph_vec::read_from_database()
{
	parent_window->get_general_data("i0",&i0);
	parent_window->get_general_data("i1",&i1);

	imax = ((nsl_vector*) layer_ptr)->get_imax();

	parent_window->get_general_data("vec_type",&vec_type);

	if (i0 < 0) 
	{
		i0 = 0;
		parent_window->set_general_data("i0",i0);
	}
	if (i1 > imax-1) 
	{
		i1 = imax-1;
		parent_window->set_general_data("i1",i1);
	}
	if (i1 < i0)
	{
		i0 = 0;
		i1 = imax-1;
		parent_window->set_general_data("i0",i0);
		parent_window->set_general_data("i1",i1);
	}

	if (vec_type == 0)
	   parent_window->set_win_sector(0,i0,0,i1,1,imax); // update win sector
	else
	   parent_window->set_win_sector(i0,0,i1,0,imax,1); // update win sector
	area_level_graph::read_from_database(); // read in new win sector
}
void area_level_graph_vec::write_to_database()
{
	area_level_graph::write_to_database();
}
void area_level_graph_vec::input(int x_pos,int y_pos,int fg)
{
	float val;
	int i,ni;

	num_type* m0 = ((nsl_vector*) layer0_ptr)->get_vector();
	num_type* m = ((nsl_vector*) layer_ptr)->get_vector();

	if (fg == 1)
	   val = 0.0;
	else
	   parent_window->get_general_data("val_in",&val);

	if (y_pos <= wy1 && y_pos >= wy0 && x_pos <= wx1 && x_pos >= wx0)
	{
	    if (vec_type == 0)
	    {
	       for (i = wx0, ni = i0 ; i + wxs <= x_pos; ni++, i = i + wxs);
    	       draw_area_level_val(canvas,i,wy0,wxs,wys,val,wymin,wymax);
	    }
	    else
	    {
	    	for (i = wy0, ni = i0 ; i + wys <= y_pos; ni++, i = i + wys);
    	    	draw_area_level_val(canvas,wx0,i,wxs,wys,val,wymin,wymax);
	    }
	    m0[ni] = val;
	    m[ni] = val;
	}
}
void area_level_graph_vec::output(int x_pos,int y_pos)
{
	float val;
	int i,ni;

	if (y_pos <= wy1 && y_pos >= wy0 && x_pos <= wx1 && x_pos >= wx0)
	{
	    if (vec_type == 0)
	    {
	    	for (i = wx0, ni = i0 ; i + wxs <= x_pos; ni++, i = i + wxs);
	    }
	    else
	    {
	    	for (i = wy0, ni = i0 ; i + wys <= y_pos; ni++, i = i + wys);
	    }

	    if (ni < imax)
	    {
	    	val = ((nsl_vector*) layer_ptr)->elem(ni);

	    	parent_window->set_general_data("elem_i",ni);
	    	parent_window->set_general_data("val_out",val);
	    }
	}
}
void area_level_graph_vec::display()
{
	int fg; // redraw fg

	num_type* m0 = ((nsl_vector*) layer0_ptr)->get_vector();
	num_type* m = ((nsl_vector*) layer_ptr)->get_vector();

	if ((fg = parent_canvas->get_redraw_fg()) == 1)
		parent_canvas->set_redraw_fg(0);

	draw_area_level_vector(canvas,m,m0,wx0,wy0,wxs,wys,
				i0,i1,wymin,wymax,fg,vec_type);

	for (int i = 0; i < imax; i++)
		m0[i] = m[i];
}
void area_level_graph_vec::print(ostream& out)
{
	area_level_graph::print(out);
}

// mat 
area_level_graph_mat::area_level_graph_mat(display_window* window,
	nsl_matrix* m) : area_level_graph(window) 
{
	layer_ptr = m;

	imax = ((nsl_matrix*) layer_ptr)->get_imax();
	jmax = ((nsl_matrix*) layer_ptr)->get_jmax();

	layer0_ptr = new nsl_matrix(imax,jmax);
}
void area_level_graph_mat::read_from_database()
{
	parent_window->get_general_data("i0",&i0);
	parent_window->get_general_data("i1",&i1);
	parent_window->get_general_data("j0",&j0);
	parent_window->get_general_data("j1",&j1);

	imax = ((nsl_matrix*) layer_ptr)->get_imax();
	jmax = ((nsl_matrix*) layer_ptr)->get_jmax();

	if (i0 < 0) 
	{
		i0 = 0;
		parent_window->set_general_data("i0",i0);
	}
	if (i1 > imax-1) 
	{
		i1 = imax-1;
		parent_window->set_general_data("i1",i1);
	}
	if (i1 < i0)
	{
		i0 = 0;
		i1 = imax-1;
		parent_window->set_general_data("i0",i0);
		parent_window->set_general_data("i1",i1);
	}

	if (j0 < 0) 
	{
		j0 = 0;
		parent_window->set_general_data("j0",j0);
	}
	if (j1 > jmax-1) 
	{
		j1 = jmax-1;
		parent_window->set_general_data("j1",j1);
	}
	if (j1 < j0)
	{
		j0 = 0;
		j1 = jmax-1;
		parent_window->set_general_data("j0",j0);
		parent_window->set_general_data("j1",j1);
	}

	parent_window->set_win_sector(i0,j0,i1,j1,imax,jmax); // update win sector
	area_level_graph::read_from_database(); // read in new win sector
}
void area_level_graph_mat::write_to_database()
{
	area_level_graph::write_to_database();
}
void area_level_graph_mat::input(int x_pos,int y_pos,int fg)
{
	float val;
	int i,j,ni,nj;

	num_type** m0 = ((nsl_matrix*) layer0_ptr)->get_matrix();
	num_type** m = ((nsl_matrix*) layer_ptr)->get_matrix();

	if (fg == 1)
	   val = 0.0;
	else
	   parent_window->get_general_data("val_in",&val);

	if (y_pos <= wy1 && y_pos >= wy0 && x_pos <= wx1 && x_pos >= wx0)
	{
// x,y crossed
//	    for (i = wx0, ni = i0; i + wxs <= x_pos; ni++, i = i + wxs);
//	    for (j = wy0, nj = j0; j + wys <= y_pos; nj++, j = j + wys);
	    for (i = wx0, nj = j0; i + wxs <= x_pos; nj++, i = i + wxs);
	    for (j = wy0, ni = i0; j + wys <= y_pos; ni++, j = j + wys);

    	    draw_area_level_val(canvas,i,j,wxs,wys,val,wymin,wymax);

	    m0[ni][nj] = val;
	    m[ni][nj] = val;
	}
}
void area_level_graph_mat::output(int x_pos,int y_pos)
{
	float val;
	int i,j,ni,nj;

	if (y_pos <= wy1 && y_pos >= wy0 && x_pos <= wx1 && x_pos >= wx0)
	{
// x,y crossed
//	    for (i = wx0, ni = i0 ; i + wxs <= x_pos; ni++, i = i + wxs);
//	    for (j = wy0, nj = j0; j + wys <= y_pos; nj++, j = j + wys);
	    for (i = wx0, nj = j0; i + wxs <= x_pos; nj++, i = i + wxs);
	    for (j = wy0, ni = i0; j + wys <= y_pos; ni++, j = j + wys);

	    if (ni < imax && nj < jmax)
	    {
	    	val = ((nsl_matrix*) layer_ptr)->elem(ni,nj);

	    	parent_window->set_general_data("elem_i",ni);
	    	parent_window->set_general_data("elem_j",nj);
	    	parent_window->set_general_data("val_out",val);
	    }
	}
}
void area_level_graph_mat::display()
{
	int fg; // redraw fg

	num_type** m0 = ((nsl_matrix*) layer0_ptr)->get_matrix();
	num_type** m = ((nsl_matrix*) layer_ptr)->get_matrix();

	if ((fg = parent_canvas->get_redraw_fg()) == 1)
		parent_canvas->set_redraw_fg(0);

	draw_area_level_matrix(canvas,m,m0,wx0,wy0,wxs,wys,
						i0,i1,j0,j1,wymin,wymax,fg);

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		m0[i][j] = m[i][j];
}
void area_level_graph_mat::print(ostream& out)
{
	area_level_graph::print(out);
}

