/****************************************************************/
/*                                                            	*/
/*                       canvas_winds.c                        	*/
/*                                                            	*/
/* by Alfredo Weitzenfeld				      	*/
/*                                                            	*/
/****************************************************************/

// static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern Cms  cms;        //added by dj 3-13-96
extern int window_dump(Frame,int,int);
extern void show_frame(Frame);

//extern void apply_canvas_colors(Canvas); 

extern window_interface* NSL_INTERFACE;

Canvas create_canvas(Frame frame,int x,int y,int width,int height,int data)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// create_canvas(" << frame << "," << x << "," << y;
	    cout << "," << width << "," << height << "," << data << ")\n";
	    cout.flush();
	}

//	cmd_out("canvas width ",width);
//	cmd_out("canvas height ",height);

	return xv_create(frame, CANVAS,
		XV_X, x,
		XV_Y, y,
		XV_WIDTH, width,
		XV_HEIGHT, height,
		CANVAS_X_PAINT_WINDOW, TRUE,
	        WIN_CLIENT_DATA,data,
		CANVAS_AUTO_EXPAND, TRUE,
		CANVAS_AUTO_SHRINK, TRUE,
		OPENWIN_AUTO_CLEAR, TRUE, 		
                CANVAS_RETAINED, TRUE,
		XV_SHOW,TRUE,
		WIN_CMS,    cms,                 //added by dj 3-13-96
                0);
}
int resize_canvas(Canvas canvas,int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// resize_canvas(" << canvas << "," << x << "," << y;
	    cout << "," << w << "," << h << ")\n";
	    cout.flush();
	}

//	cmd_out("canvas resize width ",w);
//	cmd_out("canvas resize height ",h);

	xv_set(canvas,
		XV_X,x,
		XV_Y,y,
		XV_WIDTH,w,
		XV_HEIGHT,h,
		XV_SHOW,TRUE,
                0);

	return 1;
}
void set_canvas_key_data(Canvas canvas,int key,int data)
{
	xv_set(canvas,XV_KEY_DATA,key,data,0);
}
int get_canvas_key_data(Canvas canvas,int key)
{
	return (int) xv_get(canvas,XV_KEY_DATA,key);
}
int set_canvas_defaults(Canvas canvas)
{
//	apply_canvas_colors(canvas); // only for color
//	xv_set(canvas,CANVAS_RETAINED, TRUE,0);

	return 1;
}

extern void mouse_all_canvas_proc(Xv_Window,Event*,Notify_arg);
extern void mouse_menu_canvas_proc(Xv_Window,Event*,Notify_arg);

# define BASE 1000

void set_all_canvas_events(Canvas canvas)
{
// XV_KEY_DATA doesn't work with Xv_Window, only WIN_CLIENT_DATA
	int fr_index = (int)xv_get(canvas, XV_KEY_DATA,1);
	int win_index = (int)xv_get(canvas, XV_KEY_DATA,2);
//	int data = (int)xv_get(canvas, WIN_CLIENT_DATA);
	int data = fr_index*BASE + win_index;
        xv_set(canvas_paint_window(canvas),
		WIN_CONSUME_EVENTS, 
			WIN_MOUSE_BUTTONS, 
			LOC_MOVE,
			0,
		WIN_EVENT_PROC,mouse_all_canvas_proc,
		WIN_CLIENT_DATA, data,
		0);
}
void set_menu_canvas_events(Canvas canvas)
{
// XV_KEY_DATA doesn't work with Xv_Window, only WIN_CLIENT_DATA
	int fr_index = (int)xv_get(canvas, XV_KEY_DATA,1);
	int win_index = (int)xv_get(canvas, XV_KEY_DATA,2);
//	int data = (int)xv_get(canvas, WIN_CLIENT_DATA);
	int data = fr_index*BASE + win_index;
        xv_set(canvas_paint_window(canvas),
		WIN_CONSUME_EVENTS, 
			WIN_MOUSE_BUTTONS,
			LOC_MOVE,
			0,
		WIN_EVENT_PROC,mouse_menu_canvas_proc,
		WIN_CLIENT_DATA, data,
		0);
}

// mouse position in canvas

int get_screen_matrix(Xv_Window window,int x_pos,int y_pos,int fg) 
{
 	display_frame* frame;
	display_window* win;
	display_panel* panel;

	int index = (int) xv_get(window,WIN_CLIENT_DATA);
	int win_index = index%BASE;
	int fr_index = index/BASE;

	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	    if ((win = frame->get_display_window(win_index)) != NULL)
	    {
        	if ((panel = win->get_control_panel()) != NULL)
		    panel->write_to_database("val_in");
		win->input(x_pos,y_pos,fg);
	    }
	    else
		cmd_error("Cannot get window: ",win_index);
	}
	else
		cmd_error("Cannot get frame: ",fr_index);

	return 1;
}

int get_screen_locator(Xv_Window window,int x_pos,int y_pos)
{
 	display_frame* frame;
	display_window* win;
	display_panel* panel;

	int index = (int) xv_get(window,WIN_CLIENT_DATA);
	int win_index = index%BASE;
	int fr_index = index/BASE;

	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	    if ((win = frame->get_display_window(win_index)) != NULL)
	    {
		win->output(x_pos,y_pos);
        	if ((panel = win->get_control_panel()) != NULL)
		{
		    panel->write_to_database("val_in");
		    panel->update("elem_i"); 
		    panel->update("elem_j"); 
		    panel->update("val_out"); 
		}
	    }
	    else
		cmd_error("Cannot get window: ",win_index);
	}
	else
		cmd_error("Cannot get frame: ",fr_index);

	return 1;
}

// canvas menues (when right mouse button is down)

void dump_window_menu_notify_proc(Menu menu,Menu_item item)
{
        Menu menu0;
        Menu_item item0;
  	display_frame* frame;    	
//	display_window* window;    
	Frame frame0;
	int ch,fr_index,win_index;

        ch = (int) xv_get(menu,MENU_SELECTED);
        xv_get(item,MENU_VALUE);
        item0 = (Menu_item) xv_get(menu,XV_OWNER);
        if (item0 != 0) //dj 2-2-96
        {
                menu0 = (Menu) xv_get(item0,XV_OWNER);
                fr_index = (int) xv_get(menu0,XV_KEY_DATA,1);
                win_index = (int) xv_get(menu0,XV_KEY_DATA,2);
        }
        else
	{
                fr_index = (int) xv_get(menu,XV_KEY_DATA,1);
                win_index = (int) xv_get(menu,XV_KEY_DATA,2);
	} 

	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	  if ((frame0 = frame->get_frame()) != 0) //dj 2-2-96
		window_dump(frame0,ch,0);
	    else
		cmd_error("Cannot update window: ",win_index);	
	}
	else
		cmd_error("Cannot dump frame: ",fr_index);
} 
void update_menu_notify_proc(Menu menu,Menu_item item)
{
 	display_frame* frame;    	
	display_window* window;
   
	int fr_index = (int) xv_get(menu,XV_KEY_DATA,1); 
	int win_index = (int) xv_get(menu,XV_KEY_DATA,2); 

	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	    if ((window = frame->get_display_window(win_index)) != NULL)
	    	window->update_from_panel();
	    else
		cmd_error("Unknown update window: ",win_index);	
	}
	else
		cmd_error("Unknow update frame: ",fr_index);	
}
void control_panel_notify_proc(Menu menu,Menu_item item)
{
 	display_frame* frame;    	
	display_window* window;
	display_panel* panel;
   
	int fr_index = (int) xv_get(menu,XV_KEY_DATA,1); 
	int win_index = (int) xv_get(menu,XV_KEY_DATA,2); 

	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	    if ((window = frame->get_display_window(win_index)) != NULL
		&& (panel = window->get_control_panel()) != NULL)
	    	show_frame(panel->get_frame());
	    else
		cmd_error("Unknown control panel window: ",win_index);	
	}
	else
		cmd_error("Unknown control panel window frame: ",fr_index);	
}
Menu set_nsl_canvas_menu(Xv_Window window,int fr_index,int win_index)
{
        Menu dump_menu;
	/* aa 95/5/16 */
        Menu pop2_menu;

        dump_menu =(Menu) xv_create(NULL,MENU,
                MENU_ITEM,
                        MENU_STRING," WindowDump -> Printer",
                        0,
                MENU_ITEM,
                        MENU_STRING," WindowDump -> Raster File ",
                        0,
                MENU_ITEM,
                        MENU_STRING," WindowDump -> PostScript File ",
                        0,
                MENU_NOTIFY_PROC,dump_window_menu_notify_proc,
                XV_KEY_DATA, 1, fr_index,
		XV_KEY_DATA, 2, win_index,
                0);
	/*  b aa 95/5/16 */
	if (dump_yes) {
	  pop2_menu =  (Menu) xv_create(NULL, MENU,
                MENU_PULLRIGHT_ITEM,"WinDump",dump_menu,
                MENU_ITEM,
        	        MENU_STRING,"UpdateGraph",
		        MENU_NOTIFY_PROC,update_menu_notify_proc,
			0,
		MENU_ITEM,
			MENU_STRING,"ControlPanel",
			MENU_NOTIFY_PROC,control_panel_notify_proc,
			0,
                XV_KEY_DATA, 1, fr_index,
		XV_KEY_DATA, 2, win_index,
                0);
	} else {
	  pop2_menu =  (Menu) xv_create(NULL, MENU,
                MENU_ITEM,
        	        MENU_STRING,"UpdateGraph",
		        MENU_NOTIFY_PROC,update_menu_notify_proc,
			0,
		MENU_ITEM,
			MENU_STRING,"ControlPanel",
			MENU_NOTIFY_PROC,control_panel_notify_proc,
			0,
                XV_KEY_DATA, 1, fr_index,
		XV_KEY_DATA, 2, win_index,
                0);
	}
	return (Menu) pop2_menu;
	/*  b aa 95/5/16 */
}

Menu set_nsl_canvas_menu(Xv_Window window)
{
	int index = (int) xv_get(window, WIN_CLIENT_DATA);
	int fr_index = index/BASE;
	int win_index = index%BASE;
	return set_nsl_canvas_menu(window,fr_index,win_index);
}
void mouse_all_canvas_proc(Xv_Window window,Event* event,Notify_arg arg)
{
	int x_pos,y_pos;

	if (event_id(event) == MS_LEFT && event_is_down(event))
//		(event_is_down(event) || event_id(event) == LOC_DRAG))
	{
		x_pos = event_x(event);
		y_pos = event_y(event);
		get_screen_matrix(window,x_pos,y_pos,0);
	}
	if (event_id(event) == MS_MIDDLE && event_is_down(event))
	{
		x_pos = event_x(event);
		y_pos = event_y(event);
		get_screen_matrix(window,x_pos,y_pos,1);
	}
	if (event_id(event) == MS_RIGHT && event_is_down(event))
	{
		Menu menu = set_nsl_canvas_menu(window); // made each time
		menu_show(menu,window,event,NULL);
	}
	if (event_id(event) == LOC_MOVE)
	{
		x_pos = event_x(event);
		y_pos = event_y(event);
		get_screen_locator(window,x_pos,y_pos); //canvas
	}
}
void mouse_menu_canvas_proc(Xv_Window window,Event* event,Notify_arg arg)  
{
	if (event_id(event) == MS_RIGHT && event_is_down(event))
	{
		Menu menu = set_nsl_canvas_menu(window); // made each time
		menu_show(menu,window,event,NULL);
	}
}



