//
//	display_canvas.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern  void clear_canvas(Canvas,int,int);

extern Canvas create_canvas(Frame,int,int,int,int,int);
extern int resize_canvas(Canvas,int,int,int,int);
extern void set_canvas_key_data(Canvas,int,int);

static int FONT_WIDTH = 8;

extern nsl_model* NSL_MODEL; // global model pointer

nsl_data* get_data_obj(data_type_list*,char*);

void (*INIT_USER_DISPLAY_CANVAS)(display_canvas*); // extern graphics

// display_canvas
display_canvas::display_canvas(display_frame* fr,display_window* win,
	char* str1) : (str1,0,0,0,0,0)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::display_canvas(" << str1 << ");\n";
	    cout.flush();
	}

	parent_frame = fr;	// parent display frame
	parent_window = win;	// parent display window
	canvas = 0;		// sunview canvas
}
display_canvas::~display_canvas()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::~display_canvas()");
}

void display_canvas::resize(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::resize(" << x << "," << y;
	    cout << "," << w << "," << h << ")\n";
	    cout.flush();
	}

	X0 = x;
	Y0 = y;
	width = w;
	height = h;

	if (done_fg == 1)
		cmd_error("display_canvas.c:1 Cannot open canvas");
	else
		resize_canvas(canvas,X0,Y0,width,height);
}
void display_canvas::reset()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::reset()");

	reset_data();
}
void display_canvas::reset_data()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::reset_data()");

	int vec_type;
	graph_objects = new canvas_type_list;
	object_total = 0;
	clear_fg = 1;
	redraw_fg = 1;

	label_top_x_loc = 5;		// wx0;
	label_bot_x_loc = 5;		// wx0;
	label_top_y_loc = height - 12;	// wy1+6;
	label_bot_y_loc = 5;		// wy0-13;
}
void display_canvas::init()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::init()");

	init_data();
	set_data();
}
void display_canvas::init_data()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::init_data()");

	int vec_type;

	int graph_type = parent_window->get_graph_type();
	int layer_type = parent_window->get_layer_type();
	nsl_layer* layer = parent_window->get_layer();

	init_win_label("name",name,1);

	if (layer_type == 2) // ??
	    vec_type = ((nsl_vector*) layer)->get_vec_type();

	init_win_label("LAYER",1);
//	init_win_label("title",1);
	init_win_label("wymin",1);
	init_win_label("wymax",1);
	init_win_label("ts",1);

	switch (layer_type)
	{
	case 1: // nsl_data
		break;
	case 2: // nsl_vector
		init_win_label("i0",0);
		init_win_label("i1",0);
		init_win_label("imax",0);
		break;
	case 3: // nsl_matrix
		init_win_label("i0",0);
		init_win_label("i1",0);
		init_win_label("j0",0);
		init_win_label("j1",0);
		init_win_label("imax",0);
		init_win_label("jmax",0);
		break;
	default:
		cmd_error_exit("Unknown NSL_LAYER type");
	}

	switch (graph_type)
	{
	case 1: // area_level_graph
		init_area_level_graph(layer,layer_type);
		init_win_grid(0);
		break;
	case 2: // spatial_graph_2
		init_spatial_graph_2(layer,layer_type);
		init_win_grid(1); // 'height-y' type grid
		init_win_line(0.0);
		break;
	case 3: // spatial_graph_3
		init_spatial_graph_3(layer,layer_type);
		break;
	case 4: // temporal_graph
		init_temporal_graph(layer,layer_type);
		init_win_grid(1); // 'height-y' type grid
		init_win_line(0.0);
		init_win_label("t0",0);
		init_win_label("t1",0);
		break;
	case 5: // user_graph
		if (INIT_USER_DISPLAY_CANVAS != NULL)
			INIT_USER_DISPLAY_CANVAS(this);
		else
		{
			cmd_error("display_canvas.c:*** display_canvas::init ***");
			cmd_error("display_canvas.c:Unknow INIT_USER_DISPLAY_CANVAS function");
		}
		break;
	default:
		cmd_error("display_canvas.c:*** display_canvas::init ***");
		cmd_error_exit("Unknown GRAPH type: ",graph_type);
	}

}

void display_canvas::set_data()
{
	int vec_type;

	int graph_type = parent_window->get_graph_type();
	int layer_type = parent_window->get_layer_type();
	nsl_layer* layer = parent_window->get_layer();

	set_win_label("name");

	if (layer_type == 2) // ??
	    vec_type = ((nsl_vector*) layer)->get_vec_type();

	set_win_label("LAYER");
//	set_win_label("title");
	set_win_label("wymin");
	set_win_label("wymax");
	set_win_label("ts");

	switch (layer_type)
	{
	case 1: // nsl_data
		break;
	case 2: // nsl_vector
		set_win_label("i0");
		set_win_label("i1");
		set_win_label("imax");
		break;
	case 3: // nsl_matrix
		set_win_label("i0");
		set_win_label("i1");
		set_win_label("j0");
		set_win_label("j1");
		set_win_label("imax");
		set_win_label("jmax");
		break;
	default:
		cmd_error_exit("Unknown NSL_LAYER type");
	}

	switch (graph_type)
	{
	case 1: // area_level_graph
		break;
	case 2: // spatial_graph_2
		break;
	case 3: // spatial_graph_3
		break;
	case 4: // temporal_graph
		set_win_label("t0");
		set_win_label("t1");
		break;
	case 5: // user_graph
		if (INIT_USER_DISPLAY_CANVAS != NULL)
			INIT_USER_DISPLAY_CANVAS(this);
		else
		{
			cmd_error("display_canvas.c:*** display_canvas::set_data ***");
			cmd_error("display_canvas.c:Unknow INIT_USER_DISPLAY_CANVAS function");
		}
		break;
	default:
		cmd_error("display_canvas.c:*** display_canvas::set_data ***");
		cmd_error_exit("Unknown GRAPH type: ",graph_type);
	}
}

void display_canvas::create(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::create(" << x << "," << y;
	    cout << "," << w << "," << h << ")\n";
	    cout.flush();
	}

	X0 = x; Y0 = y; width = w; height = h;

	Frame frame = parent_frame->get_frame();
	int fr_index = parent_frame->get_index();
	if ((canvas = create_canvas(frame,X0,Y0,width,height,index)) == 0 )
		cmd_error("display_canvas.c:Can't create Display Canvas");
	else
	{	// frame ('1') and canvas ('2') index
		set_canvas_key_data(canvas,1,fr_index);
		set_canvas_key_data(canvas,2,index);
                set_done_fg(0);              //added by dj
	}
}
void display_canvas::create()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::create()");

	create(X0,Y0,width,height);
}
void display_canvas::clear()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::clear()");

	if (done_fg == 1)
		cmd_error("display_canvas.c:2 Cannot open canvas");
	else
	{
		redraw_fg = 1;
		clear_canvas(canvas,get_width(),get_height());
	}
}
// database updates
void display_canvas::update()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::update()");

	read_from_database(); // init already does this
	clear();
	display();
}
// enable
void display_canvas::enable_item(int index)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::enable_item()");

	canvas_object* item;

	if (index <= 0)
		cmd_error("display_canvas.c:Bad canvas item index: ",index); 
	else if ((item = get_canvas_item(index)) != NULL)
		item->enable();
	else
		cmd_error("display_canvas.c:Cannot get canvas item index: ",index); 
}
void display_canvas::enable_item(char* str)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::enable_item()");

	canvas_object* item;

	if ((item = get_canvas_item(str)) != NULL)
		item->enable();
	else
		cmd_error("display_canvas.c:Cannot get canvas item: ",str); 
}
void display_canvas::enable_all_item()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::enable_all_item()");

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* item;
	    while (item = next()) 
		item->enable();
	}
}
// disable
void display_canvas::disable_item(int index)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::disable_item()");

	canvas_object* item;

	if (index <= 0)
		cmd_error("display_canvas.c:Bad canvas item index: ",index); 
	else if ((item = get_canvas_item(index)) != NULL)
		item->disable();
	else
		cmd_error("display_canvas.c:Cannot get canvas item index: ",index); 
}
void display_canvas::disable_item(char* str)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::disable_item()");

	canvas_object* item;

	if ((item = get_canvas_item(str)) != NULL)
		item->disable();
	else
		cmd_error("display_canvas.c:Cannot get canvas item: ",str); 
}
void display_canvas::disable_all_item()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::disable_all_item()");

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* item;
	    while (item = next()) 
		item->disable();
	}
}
canvas_object* display_canvas::get_canvas_item(int index)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::get_canvas_item(int)");

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next()) 
		if (index == obj->get_index())
		    return obj;
	}
	else
	{
	    cmd_error("display_canvas.c:*** display_canvas::get_canvas_item ***");
	    cmd_error("display_canvas.c:NULL graph_objects");
	}
	cmd_error("display_canvas.c:NULL canvas item");
	return 0;
}
canvas_object* display_canvas::get_canvas_item(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::get_canvas_item(" << name << ")\n";
	    cout.flush();
	}

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next()) 
		if (strcmp(name,obj->get_name()) == 0)
		    return obj;
	}
	else
	{
	    cmd_error("display_canvas.c:*** display_canvas::get_canvas_item ***");
	    cmd_error("display_canvas.c:NULL graph_objects");
	}
	cmd_error("display_canvas.c:NULL canvas item");
	return 0;
}
void display_canvas::read_from_database()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::read_from_database()");

	if (done_fg == 1)
		cmd_error("display_canvas.c:3 Cannot open canvas");
	else if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next()) 
		obj->read_from_database();
	}
	else
	{
	    cmd_error("display_canvas.c:*** display_canvas::read_from_database ***");
	    cmd_error("display_canvas.c:NULL graph_objects");
	}
}
void display_canvas::write_to_database()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::write_to_database()");

	if (done_fg == 1)
		cmd_error("display_canvas.c:4 Cannot open canvas");
	else if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next()) 
		obj->write_to_database();
	}
	else
	{
	    cmd_error("display_canvas.c:*** display_canvas::write_to_database ***");
	    cmd_error("display_canvas.c:NULL graph_objects");
	}
}

// display
void display_canvas::display()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::display()");

	if (done_fg == 1)
		cmd_error("display_canvas.c:5 Cannot open canvas");
	else if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next()) 
		obj->display();
	}
}

// canvas labels
int display_canvas::init_win_label(char* name,int loc)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_win_label(" << name;
	    cout << "," << loc << ");\n";
	    cout.flush();
	}

	string str;
	sprintf(str,"%s:",name);
	return init_win_label(name,str,loc);
}
int display_canvas::init_win_label(char* name,char* label,int loc)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_win_label(" << name;
	    cout << "," << label << "," << loc << ");\n";
	    cout.flush();
	}

	int index = 0;
	text_object* obj; 
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
	    index = ++object_total;
	    if ((obj = new text_object(parent_window,name,index)) != NULL)
	    {
		obj->set_loc_fg(loc); // 1 - top, 0 - bottom
		obj->set_label(label);
		graph_objects->append(obj);
	    }
	    else
	    {
		cmd_error("display_canvas.c:*** display_canvas::init_win_label ***");
		cmd_error("display_canvas.c:Cannot create canvas text object: ",name);
	    }
	}

	return index;
}
int display_canvas::set_win_label(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::set_win_label(" << name << ");\n";
	    cout.flush();
	}
		
	string str;
	text_object* obj;
	int loc;

	if ((obj = (text_object*) get_canvas_item(name)) != NULL)
	{
	    obj->read_from_database(); // update text value and size
	    sprintf(str,"%s",obj->get_text());
	    if ((loc = obj->get_loc_fg()) == 1)
	    {
		obj->set_wx(label_top_x_loc);
		obj->set_wy(label_top_y_loc);
		label_top_x_loc = label_top_x_loc + (strlen(str)+2)*FONT_WIDTH;
	    }
	    else 
	    {
		obj->set_wx(label_bot_x_loc);
		obj->set_wy(label_bot_y_loc);
		label_bot_x_loc = label_bot_x_loc + (strlen(str)+2)*FONT_WIDTH;
	    }
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::set_win_label ***");
		cmd_error("display_canvas.c:Cannot create canvas text object: ",name);
	}

	return obj->get_index();
}

// canvas line
void display_canvas::init_win_line(float val)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_win_line(" << val << ");\n";
	    cout.flush();
	}

	line_object* obj; 
	if ((obj = new line_object(parent_window,val,val)) != NULL)
	{
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_win_line ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_win_line(float val0, float val1)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_win_line(" << val0;
	    cout << "," << val1 << ");\n";
	    cout.flush();
	}

	line_object* obj; 
	if ((obj = new line_object(parent_window,val0,val1)) != NULL)
	{
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_win_line ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}

// canvas grid
void display_canvas::init_win_grid()
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_win_grid();\n";
	    cout.flush();
	}

	init_win_grid(0); // area_level_graph type grid
}
void display_canvas::init_win_grid(int fg)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_win_grid(" << fg << ");\n";
	    cout.flush();
	}

	grid_object* obj; 
	if ((obj = new grid_object(parent_window)) != NULL)
	{
		obj->set_index(++object_total);
		obj->set_type(fg);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_win_grid ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}

// init graphics

// area level graph
void display_canvas::init_area_level_graph(nsl_layer* ptr,int type)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_area_level_graph(";
	    cout << ptr->get_name() << "," << type << ");\n";
	    cout.flush();
	}

	switch (type)
	{
	case 1:
		init_area_level_graph((nsl_data*) ptr);
		break;
	case 2:
		init_area_level_graph((nsl_vector*) ptr);
		break;
	case 3:
		init_area_level_graph((nsl_matrix*) ptr);
		break;
	default:
		cmd_error("display_canvas.c:unknown layer type: ",type);
	}
}
void display_canvas::init_area_level_graph(nsl_data* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_area_level_graph(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	area_level_graph_par* obj;
	if ((obj = new area_level_graph_par(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_area_level_graph ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}

}
void display_canvas::init_area_level_graph(nsl_vector* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_area_level_graph(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	area_level_graph_vec* obj;
	if ((obj = new area_level_graph_vec(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_area_level_graph ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_area_level_graph(nsl_matrix* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_area_level_graph(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	area_level_graph_mat* obj;
	if ((obj = new area_level_graph_mat(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_area_level_graph ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}

// spatial graph 2
void display_canvas::init_spatial_graph_2(nsl_layer* ptr,int type)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_2(";
	    cout << ptr->get_name() << "," << type << ");\n";
	    cout.flush();
	}

	switch (type)
	{
	case 1:
		init_spatial_graph_2((nsl_data*) ptr);
		break;
	case 2:
		init_spatial_graph_2((nsl_vector*) ptr);
		break;
	case 3:
		init_spatial_graph_2((nsl_matrix*) ptr);
		break;
	default:
		cmd_error("display_canvas.c:unknown layer type: ",type);
	}
}
void display_canvas::init_spatial_graph_2(nsl_data* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_2(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	spatial_graph_2_par* obj;
	if ((obj = new spatial_graph_2_par(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_spatial_graph_2 ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_spatial_graph_2(nsl_vector* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_2(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	spatial_graph_2_vec* obj;
	if ((obj = new spatial_graph_2_vec(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_spatial_graph_2 ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_spatial_graph_2(nsl_matrix* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_2(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	spatial_graph_2_mat* obj;
	if ((obj = new spatial_graph_2_mat(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_spatial_graph_2 ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}

// spatial graph 3
void display_canvas::init_spatial_graph_3(nsl_layer* ptr,int type)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_3(";
	    cout << ptr->get_name() << "," << type << ");\n";
	    cout.flush();
	}

	switch (type)
	{
	case 1:
		init_spatial_graph_3((nsl_data*) ptr);
		break;
	case 2:
		init_spatial_graph_3((nsl_vector*) ptr);
		break;
	case 3:
		init_spatial_graph_3((nsl_matrix*) ptr);
		break;
	default:
		cmd_error("display_canvas.c:unknown layer type: ",type);
	}
}
void display_canvas::init_spatial_graph_3(nsl_data* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_3(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	spatial_graph_3_par* obj;
	if ((obj = new spatial_graph_3_par(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_spatial_graph_3 ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_spatial_graph_3(nsl_vector* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_3(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	spatial_graph_3_vec* obj;
	if ((obj = new spatial_graph_3_vec(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_spatial_graph_3 ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_spatial_graph_3(nsl_matrix* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_spatial_graph_3(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	spatial_graph_3_mat* obj;
	if ((obj = new spatial_graph_3_mat(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_spatial_graph_3 ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}

// temporal graph
void display_canvas::init_temporal_graph(nsl_layer* ptr,int type)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_temporal_graph(";
	    cout << ptr->get_name() << "," << type << ");\n";
	    cout.flush();
	}

	switch (type)
	{
	case 1:
		init_temporal_graph((nsl_data*) ptr);
		break;
	case 2:
		init_temporal_graph((nsl_vector*) ptr);
		break;
	case 3:
		init_temporal_graph((nsl_matrix*) ptr);
		break;
	default:
		cmd_error("display_canvas.c:unknown layer type: ",type);
	}
}
void display_canvas::init_temporal_graph(nsl_data* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_temporal_graph(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	temporal_graph_par* obj;
	if ((obj = new temporal_graph_par(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_temporal_graph ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_temporal_graph(nsl_vector* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_temporal_graph(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	temporal_graph_vec* obj;
	if ((obj = new temporal_graph_vec(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_temporal_graph ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}
void display_canvas::init_temporal_graph(nsl_matrix* ptr)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::init_temporal_graph(";
	    cout << ptr->get_name() << ");\n";
	    cout.flush();
	}

	temporal_graph_mat* obj;
	if ((obj = new temporal_graph_mat(parent_window,ptr)) != NULL)
	{
		canvas_graph = obj;
		obj->set_index(++object_total);
		graph_objects->append(obj);
	}
	else
	{
		cmd_error("display_canvas.c:*** display_canvas::init_temporal_graph ***");
		cmd_error("display_canvas.c:Unable to create canvas object");
	}
}

// mouse input
void display_canvas::input(int x_pos,int y_pos,int button)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::input(";
	    cout << x_pos << "," << y_pos << "," << button << ");\n";
	    cout.flush();
	}

	int graph_type = parent_window->get_graph_type();

	if (done_fg == 1)
		cmd_error("display_canvas.c:6 Cannot open canvas");
// virtual call
	else if (graph_type == 1)
	   ((area_level_graph*) canvas_graph)->input(x_pos,y_pos,button);
}
void display_canvas::output(int x_pos,int y_pos)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::output(";
	    cout << x_pos << "," << y_pos << ");\n";
	    cout.flush();
	}

	int graph_type = parent_window->get_graph_type();

	if (done_fg == 1)
		cmd_error("display_canvas.c:7 Cannot open canvas");
// virtual call
	else if (graph_type == 1)
	   ((area_level_graph*) canvas_graph)->output(x_pos,y_pos);
}
void display_canvas::print_status(ostream& out)
{
	if (DEBUG_FG == 1)
		cmd_out("// display_canvas::print_status(ostream&)");

	generic_window::print_status(out);

	out << "set canvas_name " << name << "\n";
//	out << "set canvas_index " << index << "\n";
//	out << "set canvas_X0 " << X0 << "\n";
//	out << "set canvas_Y0 " << Y0 << "\n";
//	out << "set canvas_width " << width << "\n";
//	out << "set canvas_height " << height << "\n";
	out << "// total canvas objects : " << object_total << "\n";
	out << "update display_canvas\n";
	out << "\n";
	out.flush();
}
void display_canvas::print_all_item_status(ostream& out)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_canvas::print_all_item_status(out)");

	char* str;
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	out << "\n// total canvas_items: " << object_total << "\n";

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next())
	    	obj->print(out);
	    out << "\nupdate display_canvas\n";
	    out.flush();
	}
	else
	    cmd_error("display_canvas.c:NULL canvas list");
}
void display_canvas::print_item_status(ostream& out,int index)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::print_item_status(out," << ",";
	    cout << index << ");\n";
	    cout.flush();
	}

	char* str;
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next())
	    {
		if (index == obj->get_index())
	    	   obj->print(out);
	    }
	    out << "\nupdate display_canvas\n";
	    out.flush();
	}
	else
	    cmd_error("display_canvas.c:NULL canvas list");
}
void display_canvas::print_item_status(ostream& out,char* str)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_canvas::print_item_status(out," << ",";
	    cout << str << ");\n";
	    cout.flush();
	}

	/* 95/2/8 amy -- char* str; */
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	if (graph_objects != NULL)
	{
	    canvas_type_iterator next(*graph_objects);
	    canvas_object* obj;
	    while (obj = next())
	    {
		if (strcmp(str,obj->get_name()) == 0)
	    	   obj->print(out);
	    }
	    out << "\nupdate display_canvas\n";
	    out.flush();
	}
	else
	    cmd_error("display_canvas.c:NULL canvas list");
}
