//
//	display_panel.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern Frame create_cmd_frame(Frame,int,int,int,int,int);
extern void set_frame_label(Frame,char*);
extern void set_frame_key_data(Frame,int,int);
extern void set_panel_key_data(Panel,int,int);

extern Panel get_cmd_panel(Frame);

extern Scrollable_panel create_scroll_panel(Frame,int,int,int,int);
extern void resize_panel(Panel panel,int,int,int,int);
extern void resize_scroll_panel(Scrollable_panel panel,int,int,int,int);
extern void show_scroll_panel(Scrollable_panel);
extern void set_panel_below(Panel,Panel);
extern void init_scrollbar(Scrollable_panel);
extern void scroll_panel_clear(Scrollable_panel);
extern void set_nsl_panel_menu(Scrollable_panel, int); /*set menu on panel*/

nsl_data* get_data_obj(data_type_list*,char*);

void (*INIT_USER_DISPLAY_PANEL)(display_panel*); // extern graphics

// display panel
display_panel::display_panel(display_frame* fr,display_window* win,char* str) 
	: (str,0,0,0,0,0)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::display_panel(" << str << ")\n";
	    cout.flush();
	}

	parent_frame = fr;	// parent display frame
	parent_window = win;	// parent window
	main_panel = 0;		// main panel
	data_panel = 0;		// data panel
}

display_panel::~display_panel()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::~display_panel()");
}
void display_panel::reset()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::reset()");

	item_list = new panel_type_list;
	item_total = 0;
}
void display_panel::reset_data()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::reset_data()");

//	data_item_list = new panel_type_list;
//	data_item_total = 0;

	reset_data_list();

	data_item_x_loc = 0;		// reset item panel location
	data_item_y_loc = 0;		// reset item panel location
}
void display_panel::reset_main()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::reset_main()");

//	main_item_list = new panel_type_list;
	main_item_total = 0;

	main_item_x_loc = 0;		// reset main item panel location
	main_item_y_loc = 0;		// reset main item panel location
}
void display_panel::reset_data_list()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::reset_data_list()");

	item_total = main_item_total; // partial reset

	if (item_list != NULL)
	{
	    // iterate main_item_elemtns & delete the rest
	    item_list->clear(main_item_total,item_total-main_item_total);
	}
	else
	{
	    cmd_error("display_panel.c:*** display_panel::reset_data_list ***");
	    cmd_error("display_panel.c:NULL list");
	}
}
void display_panel::init()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::init()");
}
void display_panel::init_main()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::init_main()");

	init_button_entry("UpdateGraph",1);
	init_data_menu_entry("LAYER",1,1);
	init_data_menu_entry("GRAPH",2,1);
}
void display_panel::init_data()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::init_data()");

	clear();

//	init_data_entry("title");
	init_data_entry("wymin");
	init_data_entry("wymax");
//	init_data_entry("ts");

	int layer_type = parent_window->get_layer_type();
	switch (layer_type)
	{
	case 1: // nsl_data
		break;
	case 2: // nsl_vector
		init_data_entry("i0");
		init_data_entry("i1");
		init_message_entry("imax");
//		init_data_entry("vec_type");
		init_data_menu_entry("vec_type",3,0);
		break;
	case 3: // nsl_matrix
		init_data_entry("i0");
		init_data_entry("i1");
		init_data_entry("j0");
		init_data_entry("j1");
		init_message_entry("imax");
		init_message_entry("jmax");
		break;
	default:
		cmd_error_exit("Unknown NSL_LAYER type");
	}

	int graph_type = parent_window->get_graph_type();
	
	init_data_menu_entry("window_overlay",4,0); // on-off

	switch (graph_type)
	{
	case 1: // area_level_graph
		if (layer_type == 2 || layer_type == 3)
			init_message_entry("elem_i");
		if (layer_type == 3)
			init_message_entry("elem_j");
		init_message_entry("val_out");
		init_data_entry("val_in");
		break;
	case 2: // spatial_graph_2
		init_data_menu_entry("pulsefg",4,0); // on-off
		break;
	case 3: // spatial_graph_3
		init_data_menu_entry("hiddenfg",4,0); // on-off
		init_data_entry("sx");
		init_data_entry("sy");
		init_data_entry("sz");
		init_data_entry("ax");
		init_data_entry("by");
		init_data_entry("cz");
		break;
	case 4: // temporal_graph
		init_data_entry("t0");
		init_data_entry("t1");
		init_data_menu_entry("pulsefg",4,0); // on-off
		break;
	case 5: // user_graph
		if (INIT_USER_DISPLAY_PANEL != NULL)
			INIT_USER_DISPLAY_PANEL(this);
		else
		{
			cmd_error("display_panel.c:*** display_panel::init ***");
			cmd_error("display_panel.c:Unknow INIT_USER_DISPLAY_PANEL function");
		}
		break;
	default:
		cmd_error("display_panel.c:*** display_panel::init_data ***");
		cmd_error_exit("Unknown GRAPH type",graph_type);
	}
}
void display_panel::resize(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::resize(" << x << "," << y;
	    cout << "," << w << "," << h << ")\n";
	    cout.flush();
	}

	X0 = x;
	Y0 = y;
	width = w;
	height = h;

	if (done_fg == 1)
		cmd_error("display_panel.c:1 Cannot open panel");
// x	else
// x		resize_scroll_panel(panel,x,y,w,h);
}
void display_panel::create(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::create(" << x << "," << y;
	    cout << "," << w << "," << h << ")\n";
	    cout.flush();
	}

	if (h < 200)
	   h = 200;

	X0 = x;
	Y0 = y;
	width = w;
	height = h;

	int mh = 100; 		// main_panel height
	int dh = h - mh; 	// data_panel height

	Frame fr = parent_frame->get_frame();
	int pid = parent_frame->get_index();
	if ((frame = create_cmd_frame(fr,x,y,w,h,index)) != 0 ) //dj
	{
	    set_frame_key_data(frame,1,pid);
	    set_frame_key_data(frame,2,index);
	    set_frame_label(frame,name);
	    if ((main_panel = get_cmd_panel(frame)) != 0) //dj
	    {
	        set_panel_key_data(main_panel,1,pid);
	        set_panel_key_data(main_panel,2,index);
		resize_panel(main_panel,0,0,w,mh);
	    }
	    else
	    {
		cmd_error("display_panel.c:*** display_panel::create ***");
		cmd_error("display_panel.c:Cannot create main panel");
	    }
	    if ((data_panel = create_scroll_panel(frame,0,mh,w,dh)) != 0) //dj
	    {
	        set_panel_key_data(data_panel,1,pid);
	        set_panel_key_data(data_panel,2,index);
		init_scrollbar(data_panel);
	    }
	    else
	    {
		cmd_error("display_panel.c:*** display_panel::create ***");
		cmd_error("display_panel.c:Cannot create scroll panel");
	    }
	}
	else
	{
		cmd_error("display_panel.c:*** display_panel::create ***");
		cmd_error("display_panel.c:Cannot create command frame");
	}
}
void display_panel::create()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::create()");

	create(X0,Y0,width,height);
}
void display_panel::init_items()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::init_items()");

	if (done_fg == 1)
		cmd_out("display_panel.c:2 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
		item->init();
	}
}
void display_panel::display()
{
	if (DEBUG_FG == 1)
	    cmd_out("display_panel.c:// display_panel::display()");

	if (done_fg == 1)
		cmd_out("display_panel.c:3 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
		item->display();
	}
}
void display_panel::display(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::display(" << name << ")\n";
	    cout.flush();
	}

	if (done_fg == 1)
		cmd_out("display_panel.c:4 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
	    	if (strcmp(name,item->get_name()) == 0) 
		    item->display();
	}
}
void display_panel::update()
{
	if (DEBUG_FG == 1)
	    cmd_out("display_panel.c:// display_panel::update()");

	if (done_fg == 1)
		cmd_out("display_panel.c:5 Cannot open panel");
	else
	{
		read_from_database();
//		clear(); 	// clear data_panel
//		init_items(); 	// re-initilaize panel entries
		display();
	}
}
void display_panel::update(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::update(" << name << ")\n";
	    cout.flush();
	}

	if (done_fg == 1)
		cmd_out("display_panel.c:6 Cannot open panel");
	else
	{
		read_from_database(name);
		display(name);
	}
}
void display_panel::clear() // clear data_panel
{
	if (DEBUG_FG == 1)
	    cmd_out("display_panel.c:// display_panel::clear()");

	if (done_fg == 1)
		cmd_out("display_panel.c:7 Cannot open panel");
	else
		scroll_panel_clear(data_panel);
}
void display_panel::init_scroll_bar() // not used in XVIEW
{
	if (DEBUG_FG == 1)
	    cmd_out("display_panel.c:// display_panel::init_scroll_bar()");

	if (done_fg == 1)
		cmd_out("display_panel.c:8 Cannot open panel");
}

// panel button
void display_panel::init_button_entry(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_button_entry(" << name << ")\n";
	    cout.flush();
	}

	init_button_entry(name,0); // data_panel
}
void display_panel::init_button_entry(char* name,int panel_fg)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_button_entry(" << name;
	    cout << "," << panel_fg << ")\n";
	    cout.flush();
	}

	char label[NAME_SIZE];
	panel_button_item_obj* obj;
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

//	if ((win_data = get_data_obj(data_list,name)) != NULL)
//	{
	    index = ++item_total;
	    sprintf(label,"%s",name);
	    if ((obj = new panel_button_item_obj(parent_window,name,index)) != NULL)
	    {
		obj->set_panel_fg(panel_fg);
	        if (panel_fg == 1)
		{
		    ++main_item_total;
		    obj->set_wx(main_item_x_loc);
		    obj->set_wy(main_item_y_loc++);
		}
		else
		{
		    obj->set_wx(data_item_x_loc);
		    obj->set_wy(data_item_y_loc++);
		}
		obj->set_label(label);
		item_list->append(obj);
		obj->init();
//		obj->read_from_database();
	    }
	    else
	    {
		cmd_error("display_panel.c:*** display_panel::init_button_entry ***");
		cmd_error("display_panel.c:Cannot create panel item: ",name);
	    }
//	}
}
// panel data
void display_panel::init_data_entry(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_data_entry(" << name << ")\n";
	    cout.flush();
	}

	init_data_entry(name,0);
}
void display_panel::init_data_entry(char* name,int panel_fg)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_data_entry(" << name;
	    cout << "," << panel_fg << ")\n";
	    cout.flush();
	}

	char label[NAME_SIZE];
	panel_data_item_obj* obj;
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
	    index = ++item_total;
	    sprintf(label,"%s:",name);
	    if ((obj = new panel_data_item_obj(parent_window,name,index)) != NULL)
	    {
		obj->set_panel_fg(panel_fg);
	        if (panel_fg == 1)
		{
		    ++main_item_total;
		    obj->set_wx(main_item_x_loc);
		    obj->set_wy(main_item_y_loc++);
		}
		else
		{
		    obj->set_wx(data_item_x_loc);
		    obj->set_wy(data_item_y_loc++);
		}
		obj->set_label(label);
		item_list->append(obj);
		obj->init();
//		obj->read_from_database(); // update value
	    }
	    else
	    {
		cmd_error("display_panel.c:*** display_panel::init_data_entry ***");
		cmd_error("display_panel.c:Cannot create panel item: ",name);
	    }
	}
}
void display_panel::init_data_menu_entry(char* name,int fg)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_data_menu_entry(" << name;
	    cout << "," << fg << ")\n";
	    cout.flush();
	}

	init_data_menu_entry(name,fg,0);
}
void display_panel::init_data_menu_entry(char* name,int fg,int panel_fg)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_data_menu_entry(" << name;
	    cout << "," << fg << "," << panel_fg << ")\n";
	    cout.flush();
	}

	char label[NAME_SIZE];
	panel_data_menu_item_obj* obj;
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
	    index = ++item_total;
	    sprintf(label,"%s:",name);
	    if ((obj = new panel_data_menu_item_obj
		(parent_window,name,index)) != NULL)
	    {
		obj->set_panel_fg(panel_fg);
		if (panel_fg == 1)
		{
		    ++main_item_total;
		     obj->set_wx(main_item_x_loc);
		     obj->set_wy(main_item_y_loc++);
		}
		else
		{
		     obj->set_wx(data_item_x_loc);
		     obj->set_wy(data_item_y_loc++);
		}
		item_list->append(obj);
		obj->set_label(label);
		obj->set_menu_index(fg);
		obj->init();
//		obj->read_from_database(); // update value
	    }
	    else
	    {
		cmd_error("display_panel.c:*** display_panel::init_data_menu_entry ***");
		cmd_error("display_panel.c:Cannot create panel item: ",name);
	    }
	}
}
void display_panel::init_message_entry(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_message_entry(" << name << ")\n";
	    cout.flush();
	}

	init_message_entry(name,0);
}
void display_panel::init_message_entry(char* name,int panel_fg)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::init_message_entry(" << name;
	    cout << "," << panel_fg << ")\n";
	    cout.flush();
	}

	char label[NAME_SIZE];
	panel_message_item_obj* obj;
	nsl_data* win_data;

	data_type_list* data_list = parent_window->get_data_list();

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
	    index = ++item_total;
	    sprintf(label,"%s:",name);
	    if ((obj = new panel_message_item_obj
		(parent_window,name,index)) != NULL)
	    {
		obj->set_panel_fg(panel_fg);
	        if (panel_fg == 1)
		{
		    ++main_item_total;
		    obj->set_wx(main_item_x_loc);
		    obj->set_wy(main_item_y_loc++);
		}
		else
		{
		    obj->set_wx(data_item_x_loc);
		    obj->set_wy(data_item_y_loc++);
		}
		item_list->append(obj);
		obj->set_label(label);
		obj->init();
//		obj->read_from_database(); // update value
	    }
	    else
	    {
		cmd_error("display_panel.c:*** display_panel::init_message_entry ***");
		cmd_error("display_panel.c:Cannot create panel item: ",name);
	    }
	}
}

panel_object* display_panel::get_panel_item(int index)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::get_panel_item(" << index << ")\n";
	    cout.flush();
	}

	if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
		if (index == item->get_index())
		    return item;
	}
	else
	{
	    cmd_error("display_panel.c:*** display_panel::get_panel_item ***");
	    cmd_error("display_panel.c:NULL item_list");
	}
	return 0;
}
panel_object* display_panel::get_panel_item(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::get_panel_item(" << name << ")\n";
	    cout.flush();
	}

	if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
		if (strcmp(name,item->get_name()) == 0)
		    return item;
	}
	else
	{
	    cmd_error("display_panel.c:*** display_panel::get_panel_item ***");
	    cmd_error("display_panel.c:NULL item_list");
	}
	return 0;
}
void display_panel::read_from_database()
{
	if (DEBUG_FG == 1)
	    cmd_out("display_panel.c:// display_panel::read_from_database()");

	if (done_fg == 1)
		cmd_out("display_panel.c:9 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
		item->read_from_database();
	}
	else
	{
	    cmd_error("display_panel.c:*** display_panel::read_from_database ***");
	    cmd_error("display_panel.c:NULL item_list");
	}
}
void display_panel::write_to_database()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::write_to_database()");

	if (done_fg == 1)
		cmd_out("display_panel.c:10 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next()) 
		item->write_to_database();
	}
	else
	{
	    cmd_error("display_panel.c:*** display_panel::write_to_database ***");
	    cmd_error("display_panel.c:NULL item_list");
	}
}
void display_panel::read_from_database(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::read_from_database(" << name << ")\n";
	    cout.flush();
	}

	if (done_fg == 1)
		cmd_out("display_panel.c:11 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next())
	    	if (strcmp(name,item->get_name()) == 0) 
		    item->read_from_database();
	}
}
void display_panel::write_to_database(char* name)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::write_to_database(" << name << ")\n";
	    cout.flush();
	}

	if (done_fg == 1)
		cmd_out("display_panel.c:12 Cannot open panel");
	else if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item;
	    while (item = next())
	    	if (strcmp(name,item->get_name()) == 0) 
		    item->write_to_database();
	}
}

void display_panel::print_status(ostream& out)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::print_status(ostream&)");

	generic_window::print_status(out);

	out << "set panel_name " << name << "\n";
//	out << "set panel_index " << index << "\n";
//	out << "set panel_X0 " << X0 << "\n";
//	out << "set panel_Y0 " << Y0 << "\n";
//	out << "set panel_width " << width << "\n";
//	out << "set panel_height " << height << "\n";
	out << "// total panel objects : " << item_total << "\n";
	out << "update display_panel\n";
	out << "\n";
	out.flush();
}
void display_panel::print_all_item_status(ostream& out)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_panel::print_all_item_status(out)");

	out << "\n// total panel_items: " << item_total << "\n";

	if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item_obj;
	    while (item_obj = next())
		item_obj->print(out);
	    out << "\nupdate display_panel\n";
	    out.flush();
	}
	else
	    cmd_error("display_panel.c:NULL panel list");
}
void display_panel::print_item_status(ostream& out,int index)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::print_item_status(out," << ",";
	    cout << index << ");\n";
	    cout.flush();
	}

	if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item_obj;
	    while (item_obj = next())
	    {
		if (index == item_obj->get_index())
		    item_obj->print(out);
	    }
	    out << "\nupdate display_panel\n";
	    out.flush();
	}
	else
	    cmd_error("display_panel.c:NULL panel list");
}
void display_panel::print_item_status(ostream& out,char* str)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_panel::print_item_status(out," << ",";
	    cout << str << ");\n";
	    cout.flush();
	}

	if (item_list != NULL)
	{
	    panel_type_iterator next(*item_list);
	    panel_object* item_obj;
	    while (item_obj = next())
	    {
		if (strcmp(str,item_obj->get_name()) == 0)
		    item_obj->print(out);
	    }
	    out << "\nupdate display_panel\n";
	    out.flush();
	}
	else
	    cmd_error("display_panel.c:NULL panel list");
}
