//
//   interpolation.c
//

# include <stdio.h>
# include <math.h>
# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;


// _______________________ AVERAGE EXACT METHOD ________________________

// V is membrane potential
// S is current input
// Prev is previous input
// tm is time constatnt

nsl_matrix diff_ae(nsl_matrix& V,nsl_matrix& S,nsl_matrix& Prev,nsl_data& tm)
{
        static float dt,tc,temp;
        int Vmax = V.get_imax();    // Size of the matrices.
         
        if (NSL_MODEL != NULL)
                dt = NSL_MODEL->get_delta();
        else
        {   
                cmd_error("*** AVERAGE EXACT METHOD ***");
                cmd_error_exit("Unknown Model");
        }

        if (dt == 0 || tm.get_data() == 0)
        {
                cmd_error("*** AVERAGE EXACT METHOD ***");
                cmd_error("delta: ",dt);
                cmd_error("time constant: ",tm.get_data());
                cmd_error_exit("Bad Integration Parameters");
        }



/* ******************* This is the real Calculation *************** */
 
        nsl_matrix term1 (Vmax,Vmax);
        nsl_matrix term2 (Vmax,Vmax);

//	static nsl_matrix Prev;

        tc = tm.get_data(); // time constant.
        temp = exp(-dt/tc);
 

        term1 = (nsl_matrix) ( (1 - temp) * S + temp * V ) ;
        term2 = (nsl_matrix) ( (S - Prev) * (temp * (tc + dt) - tc) / dt);

	Prev = S; // previous input equal current input (for each layer)

        return (nsl_matrix) (term1 + term2);
 
 
} 

