/**************************************************************/
/*                                                            */
/*              	 nsl_command_create.c                 */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

# include "nsl_command.h"

extern void set_window_location(display_window*);
extern void update_display_window(display_window*,int);
extern void update_display_frame(display_frame*,int);

extern void update_block_stim(nsl_stim_obj*);
extern void update_icon_stim(nsl_stim_obj*);
extern void update_time_interval(nsl_stim_obj*);

extern int reset_input_stim_list(nsl_layer*);
extern int reset_window_interface(window_interface*);

extern void set_window_defaults(display_window*);

nsl_stim_obj* create_block_stim(nsl_layer* layer)
{
	int type;
	nsl_stim_obj* stim;

	if (layer != NULL)
	{
	   type = layer->get_layer_type();
	   switch (type)
	   {
	   case 1:  // create stim
	      if ((stim = ((nsl_input_data*) layer)->init_block_stim()) != NULL)
			update_block_stim(stim);
	   	else
			cmd_out("Cannot create stimulus");
	   	break;
	   case 2:
	      if ((stim = ((nsl_input_vector*) layer)->init_block_stim()) != NULL)
			update_block_stim(stim);
		   else
			cmd_out("Cannot create stimulus");
		   break;
	   case 3:
	      if ((stim = ((nsl_input_matrix*) layer)->init_block_stim()) != NULL)
			update_block_stim(stim);
	   	else
			cmd_out("Cannot create stimulus");
		break;
	   default:
		cmd_out("**** create block stim ****");
	   	cmd_out("Unknown input layer type",type);
	   }
	}

	return stim;
}
nsl_stim_obj* create_icon_stim(nsl_layer* layer)
{
	int type;
	nsl_stim_obj* stim;

	if (layer != NULL)
	{
	   type = layer->get_layer_type();
	   switch (type)
	   {
	   case 1:  // create stim
	       if ((stim = ((nsl_input_data*) layer)->init_icon_stim()) != NULL)
			update_icon_stim(stim);
	        else
			cmd_out("Cannot create stimulus");
	   	break;
	   case 2:
	       if ((stim = ((nsl_input_vector*) layer)->init_icon_stim()) != NULL)
			update_icon_stim(stim);
	   	else
			cmd_out("Cannot create stimulus");
		   break;
	   case 3:
	       if ((stim = ((nsl_input_matrix*) layer)->init_icon_stim()) != NULL)
			update_icon_stim(stim);
	        else
			cmd_out("Cannot create stimulus");
	        break;
	   default:
		cmd_out("**** create icon stim ****");
	        cmd_out("Unknown input layer type",type);
	   }
	}

	return stim;
}
void create_time_stim(nsl_stim_obj* stim)
{
	update_time_interval(stim);
}
void nsl_help_create()
{
	cout << "\ncreate 'type' ['option']\n";
	cout << "\n";
	cout << "create object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "stim_list\t\tcreate new stimuli list\n";
	cout << "block_stim\t\tcreate new block stimulus\n";
	cout << "icon_stim\t\tcreate new icon stimulus\n";
	cout << "time_stim\t\tcreate stimulus time interval\n";
	cout << "window_interface\tcreate new window interface\n";
	cout << "display_frame\t\tcreate new display frame\n";
	cout << "display_window\t\tcreate new display window\n";
//	cout << "display_panel\n";
//	cout << "panel_data_entry\n";
}
int nsl_create_command(istream& in_file)
{
	int status = 1,type;
	string str1,str2,str3;
	nsl_data* data;
	nsl_stim_obj* input_stim;
	display_frame* frame;
	display_window* window;
	display_canvas* canvas;
	display_panel* panel;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_create();
		return 1;
	}

	set_global_pointers();

	if (strcmp(str1,"stim_list") == 0)
	{
		status = reset_input_stim_list(NSL_INPUT_LAYER);
	}
	else if (strcmp(str1,"block_stim") == 0)
	{
	    if (NSL_INPUT_LAYER != NULL)
	    {
		if ((input_stim = create_block_stim(NSL_INPUT_LAYER)) != NULL)
		   NSL_INPUT_STIM = input_stim;
		else
		   cmd_error("NULL input stim");
    	    }
	    else
		cmd_error("NULL input layer");
	}
	else if (strcmp(str1,"icon_stim") == 0)
	{
	    if (NSL_INPUT_LAYER != NULL)
	    {
		if ((input_stim = create_icon_stim(NSL_INPUT_LAYER)) != NULL)
		   NSL_INPUT_STIM = input_stim;
		else
		   cmd_error("NULL input stim");
    	    }
	    else
		cmd_error("NULL input layer");
	}
	else if (strcmp(str1,"time_stim") == 0)
	{
	    if (NSL_INPUT_STIM != NULL)
	    {
		create_time_stim(NSL_INPUT_STIM);
    	    }
	    else
		cmd_error("NULL input stim");
	}
	else if (strcmp(str1,"window_interface") == 0)
	{
		status = reset_window_interface(NSL_INTERFACE);
	}
	else if (NSL_INTERFACE == NULL)
	{
		cmd_error("NULL window interface");
		return -1;
	}
	else if (strcmp(str1,"display_frame") == 0)
	{
	    CMD_BUF->get_general_data("frame_name",str2);
	    if ((frame = NSL_INTERFACE->get_display_frame(str2)) != NULL)
	    {
	    	    CMD_BUF->set_general_data("frame_index",frame->get_index());
		    NSL_FRAME = frame;
	    	    update_display_frame(frame,0);
	    }
	    else if ((frame = new display_frame(0,str2)) != NULL)
	    {
	    	    CMD_BUF->set_general_data("frame_index",frame->get_index());
		    NSL_FRAME = frame;
		    NSL_INTERFACE->add_frame(frame);	// assign index
		    update_display_frame(frame,1);// from buffer
	    }
	    else
		    cmd_error("Unable to create new display frame",str2);
//	    if ((data = CMD_BUF->get_data("frame_auto_resize")) != NULL)
//		data->set_value("OFF");
	}
	else if (strcmp(str1,"display_window") == 0)
	{
	    CMD_BUF->get_general_data("window_name",str2);
	    if (NSL_FRAME != NULL)
	    {
	        if ((window = NSL_INTERFACE->get_display_window(str2)) != NULL)
		{
	    	 CMD_BUF->set_general_data("window_index",window->get_index());
		    NSL_WINDOW = window;
	    	    update_display_window(window,0);	// update window
		    set_window_defaults(NSL_WINDOW);	// set BUF canvas & panel
		}
		else if ((window = new display_window(NSL_FRAME,str2)) != NULL)
		{
	    	 CMD_BUF->set_general_data("window_index",window->get_index());
		    NSL_WINDOW = window;
//		    NSL_FRAME->add_window(window);
	    	    update_display_window(window,1); 	// update window
		    set_window_defaults(NSL_WINDOW);	// set BUF canvas & panel
		}
		else
		    cmd_error("Unable to create new display window",str2);
	    }
	    else
		cmd_error("Cannot create display");
	}
/*	else if (strcmp(str1,"display_canvas") == 0)
//	{
//	    if (NSL_FRAME != NULL)
//	    {
//	  	if ((panel = new display_panel(frame,0,panel_name)) != NULL)
//		   update_display_panel(panel);
//		else
//		     cmd_error("Unable to open new display panel");
//    	    }
//	    else
//		cmd_error("Uninitialized Frame");
//	}
	else if (strcmp(str1,"display_panel") == 0)
//	{
//	    if (NSL_FRAME != NULL)
//	    {
//	  	if ((panel = new display_panel(frame,0,panel_name)) != NULL)
//		   update_display_panel(panel);
//		else
//		     cmd_error("Unable to open new display panel");
//    	    }
//	    else
//		cmd_error("Uninitialized Frame");
//	}
	else if (strcmp(str1,"panel_item") == 0)
	{
	    if (NSL_PANEL == NULL)
		cmd_error("Unknown Display Panel");
	    else
	    {
	    	nsl_get_str(in_file,str2);
		if ((layer = NSL_MODEL->get_layer(str2)) != NULL)
		{
//		    panel->init_data_entry((nsl_data*) layer);
//		    panel->read_from_database();
		}
		else
		    cmd_error("Unknown Layer: ",str2);
	    }
	}
*/	else
		cmd_error("Unknown Modifier: ",str1);

	return status;
}
