/**************************************************************/
/*                                                            */
/*                       nsl_command_dump.c                   */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

//static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl_command.h"

extern int save_pixwin_to_printer(Frame,int,char*,char*);
extern int save_pixwin_to_raster(Frame,char*,int);
extern int save_pixwin_to_postscript(Frame,char*,int,char*,char*);
extern int file_to_printer(char*,int);

void nsl_help_dump()
{
	cout << "\n'cmd' 'option' 'name'\n";
	cout << "\n";
	cout << "printing commands\n";
	cout << "\n";
	cout << "     cmd    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "win_dump\t\tdo a window dump on the current 'frame_name'\n";
	cout << "\t'frame_name'\tdo a window dump on the specified 'frame_name'\n";
	cout << "print_dump\t\tprint the current 'dump_file'\n";
	cout << "\t'dump_file'\tprint the specified 'dump_file'\n";
}
int window_dump(Frame frame,int file_type,int cmd_type)
{
	string str,dump_file;
	int status = 1,flag,dump_ras_index,dump_ps_index;
	nsl_data* data;
	char	*model_name,*dump_ras_file,*dump_ps_file;

// dump text
        if ((data = CMD_BUF->get_data("dump_text")) != NULL)
            strcpy(str,data->get_str_value());

	if ((data = CMD_BUF->get_data("dump_auto_name")) != NULL)
	    flag = nsl_get_on_off(data);

	if ((data = CMD_BUF->get_data("network_name")) != NULL)
	    model_name = data->get_str_value();

	if ((data = CMD_BUF->get_data("dump_file")) != NULL)
	    strcpy(dump_file,data->get_str_value());

	if ((data = CMD_BUF->get_data("dump_ras_file")) != NULL)
	    dump_ras_file = data->get_str_value();

	if ((data = CMD_BUF->get_data("dump_ps_file")) != NULL)
	    dump_ps_file = data->get_str_value();

	if ((data = CMD_BUF->get_data("dump_ras_index")) != NULL)
	{
	    dump_ras_index = data->get_int_value();
	    if (flag == 1 && file_type == 2)
	    {
		dump_ras_index++;
	    	data->set_value(dump_ras_index);
	    }
	}

	if ((data = CMD_BUF->get_data("dump_ps_index")) != NULL)
	{
	    dump_ps_index = data->get_int_value();
	    if (flag == 1 && file_type == 3)
	    {
		dump_ps_index++;
	    	data->set_value(dump_ps_index);
	    }
	}

	if (file_type == 1)
	{
	    cmd_out("Window dump into PRINTER");
            save_pixwin_to_printer(frame,cmd_type,model_name,str);
	}
	else if (file_type == 2)
	{
	    if (flag == 1) // ON
	    {
	        sprintf(dump_file,"%s.ras.%d",model_name,dump_ras_index);
		CMD_BUF->set_general_data("dump_ras_file",dump_file);
	    }
	    if (strcmp(dump_file,"") == 0)
	    {
	    	cmd_error("Unknown dump file");
	    	return status;
	    }

	    cmd_out("Window dump into RASTER file: ",dump_file);
            save_pixwin_to_raster(frame,dump_file,cmd_type);
	}
	else if (file_type == 3)
	{
	    if (flag == 1) // ON
	    {
	    	sprintf(dump_file,"%s.ps.%d",model_name,dump_ps_index);
		CMD_BUF->set_general_data("dump_ps_file",dump_file);
	    }
	    if (strcmp(dump_file,"") == 0)
	    {
	    	cmd_error("Unknown dump file");
	    	return status;
	    }

	    cmd_out("Window dump into POSTSCRIPT file: ",dump_file);
            save_pixwin_to_postscript(frame,dump_file,cmd_type,model_name,str);
	}
        else
    	    cmd_error("Unknown window dump type: ",file_type);

	CMD_BUF->set_general_data("dump_file",dump_file);

	return status;
}

int nsl_win_dump_command(istream& in_file)
{
	string str1,dump_file;
	int status = 1;
	nsl_data* data;
	int file_type = 0;
	char* dump_type;
	display_frame* frame;

	if (nsl_get_str(in_file,str1) == 0)
	{
	    if ((data = CMD_BUF->get_data("frame_name")) != NULL)
	    	strcpy(str1,data->get_str_value());
	}

	if (NSL_INTERFACE == NULL)
	{
		cmd_error("Uninitialized Interface");
		status = -1;
		return 1;
	}
	else
	{
		frame = NSL_INTERFACE->get_display_frame(str1);
		if (frame == NULL)
		{
			cmd_error("Unknown dump frame: ",str1);
			return 1;
		}
	}

	if ((data = CMD_BUF->get_data("dump_type")) != NULL)
	    dump_type = data->get_str_value();

	if (strcmp(dump_type,"PRINTER") == 0 || 
	    strcmp(dump_type,"printer") == 0)
	{
	    file_type = 1;
	}
	else if (strcmp(dump_type,"RASTER") == 0 ||
	    strcmp(dump_type,"raster") == 0 ||  strcmp(dump_type,"ras") == 0)
	{
	    file_type = 2;
	}
	else if (strcmp(dump_type,"POSTSCRIPT") == 0 ||
	    strcmp(dump_type,"ps") == 0 ||  strcmp(dump_type,"PostScript") == 0)
	{
	    file_type = 3;
	}

	status = window_dump(frame->get_frame(),file_type,1);

	return status;
}

int nsl_print_dump_command(istream& in_file)
{
	string str1,dump_file;
	int status = 1;
	nsl_data* data;
	char* str;

	if (nsl_get_str(in_file,str1) == 0)
	{
	    if ((data = CMD_BUF->get_data("dump_file")) != NULL)
	    	strcpy(dump_file,data->get_str_value());
	}
	else
		strcpy(dump_file,str1);

	if (strcmp(dump_file,"") == 0)
	{
	    cmd_error("Unknown dump file");
	    return status;
	}

      	if ((str = (char *) strrchr(dump_file,'.')) != NULL)
	{
	   if (strncmp(str,".ras",4) == 0)
	   {
		cmd_out("// Printing raster file: ",dump_file);
		file_to_printer(dump_file,1);
	   }
	   else if (strncmp(str,".ps",3) == 0)
	   {
		cmd_out("// Printing postscipt file: ",dump_file);
		file_to_printer(dump_file,2);
	   }
	   else
		cmd_error("Unknown file type: ",dump_file);
	}
	else
		cmd_error("Unknown file type: ",dump_file);

	return status;
}
