/**************************************************************/
/*                                                            */
/*                       nsl_command_file.c                   */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

//static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl_command.h"

int OUT_STREAM_FG;
fstream* OUT_STREAM;
int IN_STREAM_FG;
fstream* IN_STREAM;

extern int LOG_FILE_FG;
extern fstream* LOG_FILE;
extern nsl_file* NSL_FILE;

// check for legal file_name

extern void nsl_log_command();

extern void set_global_model_pointers(); 

int open_stream(char* file_name,int fg)
{
	int status = 1;
	fstream* fs;

	nsl_file* fnsl = new nsl_file(file_name);
	NSL_SYSTEM->add_file(fnsl);

	if (fg == 0)
	{
	    status = fnsl->open(0); // input

	    fs = fnsl->get_file_stream();

	    if (!(*fs))
	    {
		cmd_error("Cannot open file: ",file_name);
		return -1;
	    }

	    cmd_out("// Opening read file: ",file_name);

	    IN_STREAM = fs;
	    IN_STREAM_FG = 1;
	}
	else if (fg == 1)
	{
	    status = fnsl->open(1); // output

	    fs = fnsl->get_file_stream();

	    if (!(*fs))
	    {
		cmd_error("Cannot open file: ",file_name);
		return -1;
	    }

	    cmd_out("// Opening write file: ",file_name);

	    OUT_STREAM = fs;
	    OUT_STREAM_FG = 1;
	}
	else
		cmd_error("Bad file type: ",fg);

	return status;
}
int close_stream(char* file_name)
{
	int status = 1;

	nsl_file* fnsl;

	if ((fnsl = NSL_SYSTEM->get_file(file_name)) == NULL)
	{
		cmd_error("Unknown file: ",file_name);
		return(0);
	}

	int fg = fnsl->get_io_type();

	if (fnsl != NULL)
		fnsl->close();
	else
	{
		cmd_error("NULL file");
		return 1;
	}

	if (fg == 0)
	{
	    cmd_out("// Closing read file: ",fnsl->get_fname());

	    IN_STREAM = 0;
	    IN_STREAM_FG = 0;
	}
	else if (fg == 1)
	{
	    cmd_out("// Closing write file: ",fnsl->get_fname());

	    OUT_STREAM = 0;
	    OUT_STREAM_FG = 0;
	}
	else
		cmd_error("Bad file type: ",fg);

	return status;
}
int load_file(char* file_name)
{
	nsl_file* fnsl = new nsl_file(file_name);
	NSL_SYSTEM->add_file(fnsl);

	nsl_file* gnsl;

	int status = fnsl->open(0); // input

	if (status == 1)
	{
	    nsl_log_command(); // log 'load' command
	    if (LOG_FILE_FG == 1)
	    {
		gnsl = new nsl_file(file_name);
		gnsl->set_type(1); 	// log file type
		gnsl->set_dname("."); 	// log dir
		gnsl->set_name(fnsl->get_name());
		gnsl->set_fullname();
		gnsl->open(1); // output
		NSL_SYSTEM->add_file(gnsl);
		NSL_SYSTEM->push_log_file(NSL_FILE);
		NSL_FILE = gnsl;
		LOG_FILE = gnsl->get_file_stream();
	    }
	    fnsl->read();

	    fnsl->close();

	    if (LOG_FILE_FG == 1)
	    {
		gnsl->close();
		NSL_FILE = NSL_SYSTEM->pop_log_file();
		LOG_FILE = NSL_FILE->get_file_stream();
	    }
	}

	return status;
}
int write_file(char* file_name)
{
	nsl_file* fnsl = new nsl_file(file_name);

	int status = fnsl->write();

	return status;
}
int read_file(char* file_name)
{
	nsl_file* fnsl = new nsl_file(file_name);

	int status = fnsl->read();

	return status;
}
void nsl_help_file()
{
	cout << "\n'cmd' ['option'] 'name' \n";
	cout << "\n";
	cout << "   cmd    \t  option  \t    description\n";
	cout << "----------\t----------\t-------------------\n";
	cout << "\n";
	cout << "file_enable\t\tenable layer 'name' for file in/out\n";
	cout << "file_disable\t\tdisable layer 'name' for file in/out\n";
	cout << "load,read\t\t\tread from file 'name'\n";
	cout << "write\t\t\t\twrite into file 'name'\n";
	cout << "open\t\t\t\topen file 'name' according to 'file_type'\n";
	cout << "\t\tINPUT\t\topen file 'name' for input\n";
	cout << "\t\tOUTPUT\t\topen file 'name' for output\n";
	cout << "close\t\t\t\tclose file 'name'\n";
}
int nsl_file_enable_command(istream& in_file) // enable layer
{
	string str1,str2;
	int index,status = 1;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_file();
		return 1;
	}

	if (nsl_get_str(in_file,str2) == 0)
	{
		nsl_help_file();
		return 1;
	}

	if (strcmp(str1,"layer") == 0)
	{	
	    if (nsl_get_str(in_file,str2) == 0)
		NSL_LAYER->enable();
	    else if ((index = atoi(str2)) != 0)
		NSL_MODEL->enable_layer(index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_MODEL->enable_all_layer();
	    else
		NSL_MODEL->enable_layer(str2);
	}

	return status;
}
int nsl_file_disable_command(istream& in_file) // disable layer
{
	string str1,str2;
	int index,status = 1;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_file();
		return 1;
	}

	if (nsl_get_str(in_file,str2) == 0)
	{
		nsl_help_file();
		return 1;
	}

	if (strcmp(str1,"layer") == 0)
	{
	    if (nsl_get_str(in_file,str2) == 0)
		NSL_LAYER->disable();
	    else if ((index = atoi(str2)) != 0)
		NSL_MODEL->disable_layer(index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_MODEL->disable_all_layer();
	    else
		NSL_MODEL->disable_layer(str2);
	}

	return status;
}
int nsl_load_command(istream& in_file) // read
{
	string str1;
	int status = 1;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_file();
		return 1;
	}

	status = load_file(str1);

	if (status == 1)
		set_global_model_pointers(); 	// set module and layer ptrs

	return status;
}
int nsl_write_command(istream& in_file)
{
	string str1;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_file();
		return 1;
	}

	int status = write_file(str1);

	return status;
}
int nsl_open_command(istream& in_file)
{
	string str1,str2,file_name,name;
	int status = 1,flag = 0,file_index,file_type;
	nsl_data* data;
	char* str;
	int type_fg = 0; // 1 - read filename after file type

	if (nsl_get_str(in_file,str1) == 0)
	{
	    if (NSL_MODEL != NULL)
		strcpy(name,NSL_MODEL->get_name());
	    else
		strcpy(name,"TMP");
	}
	else
	{
	    if (strcmp(str1,"INPUT") == 0 || strcmp(str1,"input") == 0)
	    {
	    	file_type = 0;
		type_fg = 1;
	    }
	    else if (strcmp(str1,"OUTPUT") == 0 || strcmp(str1,"output") == 0)
	    {
	    	file_type = 1;
		type_fg = 1;
	    }
	    else
	    {
	    	strcpy(name,str1);
	    }
	}

	if (type_fg == 0 && (data = CMD_BUF->get_data("file_type")) != NULL)
	{
	        str = data->get_str_value();
	        if (strcmp(str,"INPUT") == 0)
		   file_type = 0;
	        else if (strcmp(str,"OUTPUT") == 0)
		   file_type = 1;
	        else
		{
		   cmd_error("Bad str type: ",str);
		   return 1;
		}
	}
	else if (type_fg == 1)
	{
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		nsl_help_file();
		return 1;
	    }
	    else
		strcpy(name,str2);
	}

	if ((data = CMD_BUF->get_data("file_auto_name")) != NULL)
	    flag = nsl_get_on_off(data);

	if (flag == 1 && file_type == 1) // auto name only for output
	{
	    if ((data = CMD_BUF->get_data("file_index")) != NULL)
		file_index = data->get_int_value();
	    sprintf(file_name,"%s.nsl.%d",name,file_index);
	    CMD_BUF->set_general_data("file_index",file_index+1);
	}
	else
	    strcpy(file_name,name);

	CMD_BUF->set_general_data("file_name",file_name);

	open_stream(file_name,file_type);

	return status;
}
int nsl_close_command(istream& in_file)
{
	string str1;
	int status = 1;
	nsl_data* data;
	char* file_name;

	if (nsl_get_str(in_file,str1) == 0)
	{
	    if ((data = CMD_BUF->get_data("file_name")) != NULL)
		file_name = data->get_str_value();
	}
	else
		file_name = str1;

	close_stream(file_name);

	return status;
}
