/**************************************************************/
/*                                                            */
/*                       nsl_command_proc.c                   */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

//static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl_command.h"

extern fstream* OUT_STREAM;
extern int OUT_STREAM_FG;
extern fstream* IN_STREAM;
extern int IN_STREAM_FG;

extern int BREAKFG;
int INTERRUPTFG = 0; 	// set to 1 if BREAKFG has been serviced 
			// left to the user for reset

void nsl_help_proc() // status help on the system
{
	cout << "\n'proc' ['option']\n";
	cout << "\n";
	cout << "process model\n";
	cout << "\n";
	cout << "    proc    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "init\t\t\t\tprocess modules for t=0\n";
	cout << "run\t\t\t\tprocess modules for 0<=t<=end_time\n";
	cout << "\t\t'ts'\t\tprocess modules for 0<=t<=ts\n";
	cout << "cont\t\t\t\tprocess modules for current_time<t<=end_time\n";
	cout << "\t\t'ts'\t\tprocess modules for current_time<t<=ts\n";
	cout << "step\t\t\t\tprocess modules 1 iteration\n";
	cout << "\t\t'n'\t\tprocess modules n iterations\n";
}
void nsl_help_exec() // status help on the system
{
	cout << "\nexec 'name'\n";
	cout << "\n";
	cout << "exec\t\t\t\texecute a particular module (by name)\n";
}
int nsl_iterate_step(int i0,int i1)
{
	int i;

//	if (SIM_FG == 1)

	if (i1 < i0)
	{
	    cerr << "BAD time interval: " << i0 << "," << i1 << "\n";
	    cerr.flush();
	    return 0;
	}

//	cmd_out("running from  i0 (time step): ",i0);
//	cmd_out("        until i1 (time step): ",i1);

	for (i = i0; i <= i1; i++)
	{
	    if (BREAKFG == 1)
	    {
		BREAKFG = 0;
		INTERRUPTFG = 1;
		cmd_error("Simulation Interrupt");
		return 1;
	    }
	    NSL_MODEL->set_time(i);
	    if (IN_STREAM_FG == 1 && IN_STREAM != NULL)
		(*IN_STREAM) >> NSL_MODEL;
	    NSL_MODEL->run();
	    if (OUT_STREAM_FG == 1 && OUT_STREAM != NULL)
		(*OUT_STREAM) << NSL_MODEL;
 	    if (NSL_INTERFACE != NULL)
	    {
	    	if (i == 0)
		    NSL_INTERFACE->clear();
	        NSL_INTERFACE->display();
	    }
	}
	return 1;
}
int nsl_iterate(float t0,float t1)
{
//	if (SIM_FG == 1)

	if (t1 < t0)
	{
	    cerr << "BAD time interval: " << t0 << "," << t1 << "\n";
	    cerr.flush();
	    return 0;
	}

//	cmd_out("running from  t0 (time): ",t0);
//	cmd_out("        until t1 (time): ",t1);

	float delta = NSL_MODEL->get_delta();
	int i0 = rint(t0/delta);
	int i1 = rint(t1/delta);

	int status = nsl_iterate_step(i0,i1);

	return status;
}
int nsl_init_command(istream& in_file)
{
	string str1;

	if (NSL_MODEL == NULL)
	{
	    cmd_error("NULL NSL_MODEL");
	    return 0;
	}

	int status = nsl_iterate_step(0,0);

	return status;
}
int nsl_run_command(istream& in_file)
{
	string str1;
	float t1;

	if (NSL_MODEL == NULL)
	{
	    cmd_error("NULL NSL_MODEL");
	    return 0;
	}

	if (nsl_get_str(in_file,str1) != 0)
	    t1 = atof(str1); 		// end_time
	else
	    t1 = NSL_MODEL->get_end_time();
	
	int status = nsl_iterate(0.0,t1);

	return status;
}
int nsl_cont_command(istream& in_file)
{
	string str1;
	float t1;

	if (NSL_MODEL == NULL)
	{
	    cmd_error("NULL NSL_MODEL");
	    return 0;
	}

	float t0 = NSL_MODEL->get_time() + NSL_MODEL->get_delta();

	if (nsl_get_str(in_file,str1) != 0)
	    t1 = atof(str1); 		// end_time
	else
	    t1 = NSL_MODEL->get_end_time();

	int status = nsl_iterate(t0,t1);

	return status;
}
int nsl_step_command(istream& in_file)
{
	string str1;
	int status;
	int i;

	if (NSL_MODEL == NULL)
	{
	    cmd_error("NULL NSL_MODEL");
	    return 0;
	}

	int i0 = NSL_MODEL->get_time_step() + 1;

	if (nsl_get_str(in_file,str1) != 0)
	    i = atoi(str1); 	// n iterations
	else
	    i = 1;		// 1 iteration

	int i1 = i0 + i - 1;

	status = nsl_iterate_step(i0,i1);

	return status;
}
int nsl_exec_command(istream& in_file)
{
	nsl_module* mod;
	string str1;
	int status;
	int i;

	if (NSL_MODEL == NULL)
	{
	    cmd_error("NULL NSL_MODEL");
	    return 0;
	}

	if (nsl_get_str(in_file,str1) == 0)
	{
		cmd_error("Unknown module name");
		return -1;
	}

	if ((mod = NSL_MODEL->get_module(str1)) == 0)
	{
		cmd_error("Unknown module name");
		status = -1;
	}
	else
	{
		mod->exec();
		status = 1;
	}

	return status;
}








