/**************************************************************/
/*                                                            */
/*              	 nsl_command_status.c                 */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

# include "nsl_command.h"

void nsl_help_status() // status help on the system
{
	cout << "\nstatus 'type' ['option']\n";
	cout << "\n";
	cout << "print status information for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "system\t\t\t\tlist all linked models\n";
	cout << "files\t\t\t\tlist all active data files\n";
	cout << "current\t\t\t\tcurrent system status\n";
	cout << "network\t\t\t\tstatus on current model\n"; // model
	cout << "\t\t'name'\t\tstatus on a model by name\n";
	cout << "\t\t'index'\t\tstatus on a model by index\n";
	cout << "module\t\t\t\tstatus on current module\n";
	cout << "\t\t'name'\t\tstatus on a module by name\n";
	cout << "\t\t'index'\t\tstatus on a module by index\n";
	cout << "layer\t\t\t\tstatus on current layer\n";
	cout << "\t\t'name'\t\tstatus on a layer by name\n";
	cout << "\t\t'index'\t\tstatus on a layer by index\n";
	cout << "data_value\t\t\tcurrent layer data values\n";
	cout << "\t\t'name'\t\tcurrent layer data values by name\n";
	cout << "\t\t'index'\t\tcurrent layer data values by index\n";
	cout << "input_layer\t\t\tstatus on current input_layer\n";
	cout << "\t\t'name'\t\tstatus on an input_layer by name\n";
	cout << "\t\t'index'\t\tstatus on an input_layer by index\n";
	cout << "input_stim\t\t\tprint all input layer stimuli\n";
	cout << "\t\t'name'\t\tprint stimuli of an input layer by name\n";
	cout << "\t\t'index'\t\tprint stimuli of an input layer by index\n";
	cout << "\n";
	cout << "window_interface\t\tinterface status information\n";
	cout << "display_frame\t\t\tframe status information\n";
	cout << "\t\t'name'\t\tprint display frame status by name\n";
	cout << "\t\t'index'\t\tprint display frame status by index\n";
	cout << "display_window\t\t\twindow status information\n";
	cout << "\t\t'name'\t\tprint display window status by name\n";
	cout << "\t\t'index'\t\tprint display window status by index\n";
	cout << "display_canvas\t\t\tcanvas status information\n";
	cout << "\t\t'name'\t\tprint display canvas status by name\n";
	cout << "\t\t'index'\t\tprint display canvas status by index\n";
	cout << "display_panel\t\t\tpanel status information\n";
	cout << "\t\t'name'\t\tprint display panel status by name\n";
	cout << "\t\t'index'\t\tprint display panel status by index\n";
	cout << "window_item\t\t\twindow item status information\n";
	cout << "\t\t'name'\t\tprint window item status by name\n";
	cout << "\t\t'index'\t\tprint window item status by index\n";
	cout << "canvas_item\t\t\tcanvas item status information\n";
	cout << "\t\t'name'\t\tprint canvas item status by name\n";
	cout << "\t\t'index'\t\tprint canvas item status by index\n";
	cout << "panel_item\t\t\tpanel item status information\n";
	cout << "\t\t'name'\t\tprint panel item status by name\n";
	cout << "\t\t'index'\t\tprint panel item status by index\n";
	cout << "\n";
	cout << "- 'name' is either a string name or 'ALL'\n";
	cout << "- 'index' is an integer\n";
}
void show_current_status(ostream& out)
{
	out << "\n";
	if (NSL_MODEL != NULL)
	    out << "set network " << NSL_MODEL->get_name() << "\n";
	else
	    out << "// NULL NSL_MODEL" << "\n";
	if (NSL_MODULE != NULL)
	    out << "set module " << NSL_MODULE->get_name() << "\n";
	else
	    out << "// NULL NSL_MODULE" << "\n";
	if (NSL_LAYER != NULL)
	    out << "set layer " << NSL_LAYER->get_name() << "\n";
	else
	    out << "// NULL NSL_LAYER" << "\n";
	if (NSL_INPUT_LAYER != NULL)
	    out << "set input_layer " << NSL_INPUT_LAYER->get_name() << "\n";
	else
	    out << "// NULL NSL_INPUT_LAYER" << "\n";
	if (NSL_INTERFACE != NULL)
	{
	    if (NSL_FRAME != NULL)
	    	out << "set frame " << NSL_FRAME->get_name() << "\n";
	    else
	    	out << "// NULL NSL_FRAME" << "\n";
	    if (NSL_WINDOW != NULL)
		out << "set window " << NSL_WINDOW->get_name() << "\n";
	    else
	    	out << "// NULL NSL_WINDOW" << "\n";
	    if (NSL_CANVAS != NULL)
		out << "set canvas " << NSL_CANVAS->get_name() << "\n";
	    else
	    	out << "// NULL NSL_CANVAS" << "\n";
	    if (NSL_PANEL != NULL)
		out << "set panel " << NSL_PANEL->get_name() << "\n";
	    else
	    	out << "// NULL NSL_PANEL" << "\n";
	    if (NSL_WINDOW_ITEM != NULL)
		out << "set window_item " << NSL_WINDOW_ITEM->get_index() << "\n";
	    else
	    	out << "// NULL NSL_WINDOW_ITEM" << "\n";
	    if (NSL_CANVAS_ITEM != NULL)
		out << "set canvas_item " << NSL_CANVAS_ITEM->get_index() << "\n";
	    else
	    	out << "// NULL NSL_CANVAS_ITEM" << "\n";
	    if (NSL_PANEL_ITEM != NULL)
		out << "set panel_item " << NSL_PANEL_ITEM->get_index() << "\n";
	    else
	    	out << "// NULL NSL_PANEL_ITEM" << "\n";
	}
	else
	    out << "// NULL NSL_WINDOW_INTERFACE" << "\n";
	out.flush();
}
int nsl_status_command(istream& in_file)
{
	string str1,str2,str3;
	int status = 1;
	int index;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_status();
		return 1;
	}

	if (strcmp(str1,"system") == 0)
	{
	    if (NSL_SYSTEM != NULL)
		NSL_SYSTEM->print_model_status(LOG_STREAM);
	    else
		cmd_error("NULL NSL_SYSTEM");
	}
	else if (strcmp(str1,"files") == 0)
	{
	    if (NSL_SYSTEM != NULL)
		NSL_SYSTEM->print_all_file_status(LOG_STREAM);
	    else
		cmd_error("NULL NSL_SYSTEM");
	}
	else if (strcmp(str1,"current") == 0)
	{
		show_current_status(LOG_STREAM);
	}
	else if (strcmp(str1,"model") == 0 || strcmp(str1,"network") == 0)
	{
	  if (NSL_SYSTEM != NULL)
	  {
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_MODEL != NULL)
		   NSL_MODEL->print_status(LOG_STREAM);
		else
		   cmd_error("NULL NSL_MODEL");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_SYSTEM->print_model_status(LOG_STREAM,index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_SYSTEM->print_all_model_status(LOG_STREAM);
	    else
		NSL_SYSTEM->print_model_status(LOG_STREAM,str2);
	  }
	  else
		cmd_error("NULL NSL_SYSTEM");
	}
	else if (strcmp(str1,"module") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_MODULE != NULL)
		   NSL_MODULE->print_status(LOG_STREAM);
		else
		   cmd_error("NULL NSL_MODULE");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_MODEL->print_module_status(LOG_STREAM,index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_MODEL->print_all_module_status(LOG_STREAM);
	    else
		NSL_MODEL->print_module_status(LOG_STREAM,str2);
	  }
	  else
		cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"layer") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_LAYER != NULL)
		   NSL_LAYER->print_status(LOG_STREAM);
		else
		   cmd_error("NULL NSL_LAYER");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_MODEL->print_layer_status(LOG_STREAM,index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_MODEL->print_all_layer_status(LOG_STREAM);
	    else
		NSL_MODEL->print_layer_status(LOG_STREAM,str2);
	  }
	  else
		cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"data_value") == 0)
	{
	    if (nsl_get_str(in_file,str2) == 0)
		NSL_LAYER->print(LOG_STREAM);
	    else if ((index = atoi(str2)) != 0)
		NSL_MODEL->print_layer_data(LOG_STREAM,index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_MODEL->print_all_layer_data(LOG_STREAM);
	    else
		NSL_MODEL->print_layer_data(LOG_STREAM,str2);
	}
	else if (strcmp(str1,"input_layer") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_INPUT_LAYER != NULL)
		   NSL_INPUT_LAYER->print_status(LOG_STREAM);
		else
		   cmd_error("NULL NSL_INPUT_LAYER");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_MODEL->print_input_layer_status(LOG_STREAM,index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_MODEL->print_all_input_layer_status(LOG_STREAM);
	    else
		NSL_MODEL->print_input_layer_status(LOG_STREAM,str2);
	  }
	  else
		cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_stim") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_INPUT_STIM != NULL)
		   NSL_INPUT_STIM->print_stim(LOG_STREAM);
		else
		   cmd_error("NULL NSL_INPUT_STIM");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_INPUT_LAYER->print_stim(LOG_STREAM,index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_INPUT_LAYER->print_stim(LOG_STREAM); // all
	    else
		cmd_out("Cannot print stim by name, only by index");
//		NSL_INPUT_LAYER->print_stim(LOG_STREAM,str2);
	  }
	  else
		cmd_error("NULL NSL_INPUT_LAYER");
	}
// window interface
	else if (strcmp(str1,"inderface") == 0 || 
	    strcmp(str1,"window_interface") == 0)
	{
	    if (NSL_INTERFACE != NULL)
		NSL_INTERFACE->print_status(LOG_STREAM);
	    else
	    {
		cmd_error("Uninitialized Interface");
		status = -1;
	    }
	}
	else if (strcmp(str1,"frame") == 0 || 
	    strcmp(str1,"display_frame") == 0)
	{
	    if (NSL_INTERFACE != NULL)
	    {
	    	if (nsl_get_str(in_file,str2) == 0)
		{
		   if (NSL_FRAME != NULL)
		   	NSL_FRAME->print_status(LOG_STREAM);
		    else
			cmd_error("NULL NSL_FRAME");
		}
	        else if ((index = atoi(str2)) != 0)
		   NSL_INTERFACE->print_frame_status(LOG_STREAM,index);
	        else if (strcmp("ALL",str2) == 0)
		   NSL_INTERFACE->print_frame_status(LOG_STREAM); // all
	        else
		   NSL_INTERFACE->print_frame_status(LOG_STREAM,str2);
	    }
	    else
		cmd_error("NULL NSL_INTERFACE");
	}
	else if (strcmp(str1,"window") == 0 || 
	    strcmp(str1,"display_window") == 0)
	{
	    if (NSL_FRAME != NULL)
	    {
	        if (nsl_get_str(in_file,str2) == 0)
		{
		    if (NSL_WINDOW != NULL)
		   	NSL_WINDOW->print_status(LOG_STREAM);
		    else
			cmd_error("NULL NSL_WINDOW");
		}
		else if ((index = atoi(str2)) != 0)
		   NSL_FRAME->print_window_status(LOG_STREAM,index);
	        else if (strcmp("ALL",str2) == 0)
		   NSL_FRAME->print_all_window_status(LOG_STREAM);
	        else
		   NSL_FRAME->print_window_status(LOG_STREAM,str2);
	    }
	    else
		cmd_error("NULL NSL_FRAME");
	}
	else if (strcmp(str1,"canvas") == 0 || 
	    strcmp(str1,"display_canvas") == 0)
	{
	    if (NSL_WINDOW != NULL)
	    {
	        if (nsl_get_str(in_file,str2) == 0)
		{
		   if (NSL_CANVAS != NULL)
		   	NSL_CANVAS->print_status(LOG_STREAM);
		   else
			cmd_error("NULL NSL_CANVAS");
		}
		else if ((index = atoi(str2)) != 0)
		{
		   if ((NSL_WINDOW = NSL_FRAME->get_display_window(index)) != NULL)
		   	NSL_WINDOW->print_canvas_status(LOG_STREAM);
		   else
			cmd_error("Unknown canvas: ",index);
		}
//	        else if (strcmp("ALL",str2) == 0)
//		   NSL_WINDOW->print_all_canvas_status(LOG_STREAM);
	        else
		{
		   if ((NSL_WINDOW = NSL_FRAME->get_display_window(str2)) != NULL)
		   	NSL_WINDOW->print_canvas_status(LOG_STREAM);
		   else
			cmd_error("Unknown canvas: ",str2);
		}
	    }
	    else
		cmd_error("NULL NSL_WINDOW");
	}
	else if (strcmp(str1,"panel") == 0 || 
	    strcmp(str1,"display_panel") == 0)
	{
	    if (NSL_WINDOW != NULL)
	    {
	        if (nsl_get_str(in_file,str2) == 0)
		{
		   if (NSL_PANEL != NULL)
		   	NSL_PANEL->print_status(LOG_STREAM);
		   else
			cmd_error("NULL NSL_PANEL");
		}
		else if ((index = atoi(str2)) != 0)
		{
		   if ((NSL_WINDOW = NSL_FRAME->get_display_window(index)) != NULL)
		   	NSL_WINDOW->print_panel_status(LOG_STREAM);
		   else
			cmd_error("Unknown panel: ",index);
		}
//	        else if (strcmp("ALL",str2) == 0)
//		   NSL_WINDOW->print_all_panel_status(LOG_STREAM);
	        else
		{
		   if ((NSL_WINDOW = NSL_FRAME->get_display_window(str2)) != NULL)
		   	NSL_WINDOW->print_panel_status(LOG_STREAM);
		   else
			cmd_error("Unknown panel: ",str2);
		}
	    }
	    else
		cmd_error("NULL NSL_WINDOW");
	}
	else if (strcmp(str1,"window_item") == 0)
	{
	    if (NSL_WINDOW != NULL)
	    {
	        if (nsl_get_str(in_file,str2) == 0)
		{
		    if (NSL_WINDOW_ITEM != NULL)
		        NSL_WINDOW_ITEM->print_status(LOG_STREAM);
		    else
			cmd_error("NULL NSL_WINDOW_ITEM");
		}
		else if ((index = atoi(str2)) != 0)
		   NSL_WINDOW->print_item_status(LOG_STREAM,index);
	        else if (strcmp("ALL",str2) == 0)
		   NSL_WINDOW->print_all_item_status(LOG_STREAM);
	        else
		   NSL_WINDOW->print_item_status(LOG_STREAM,str2);
	    }
	    else
		cmd_error("NULL NSL_WINDOW");
	}
	else if (strcmp(str1,"canvas_item") == 0)
	{
	    if (NSL_CANVAS != NULL)
	    {
	        if (nsl_get_str(in_file,str2) == 0)
		{
		   if (NSL_CANVAS_ITEM != NULL)
		   	NSL_CANVAS_ITEM->print(LOG_STREAM);
		   else
			cmd_error("NULL NSL_CANVAS_ITEM");
		}
		else if ((index = atoi(str2)) != 0)
		   NSL_CANVAS->print_item_status(LOG_STREAM,index);
	        else if (strcmp("ALL",str2) == 0)
		   NSL_CANVAS->print_all_item_status(LOG_STREAM);
	        else
		   NSL_CANVAS->print_item_status(LOG_STREAM,str2);
	    }
	    else
		cmd_error("NULL NSL_CANVAS");
	}
	else if (strcmp(str1,"panel_item") == 0)
	{
	    if (NSL_PANEL != NULL)
	    {
	        if (nsl_get_str(in_file,str2) == 0)
		{
		   if (NSL_PANEL_ITEM != NULL)
		   	NSL_PANEL_ITEM->print(LOG_STREAM);
		   else
			cmd_error("NULL NSL_PANEL_ITEM");
		}
		else if ((index = atoi(str2)) != 0)
		   NSL_PANEL->print_item_status(LOG_STREAM,index);
	        else if (strcmp("ALL",str2) == 0)
		   NSL_PANEL->print_all_item_status(LOG_STREAM);
	        else
		   NSL_PANEL->print_item_status(LOG_STREAM,str2);
	    }
	    else
		cmd_error("NULL NSL_PANEL");
	}
	else if (strcmp(str1,"command_buffer") == 0)
	{
	    if (CMD_BUF != NULL)
	    {
//	        if (nsl_get_str(in_file,str2) == 0)
	 	    CMD_BUF->print(LOG_STREAM);
//		else if (strcmp(str2,"canvas_item") == 0)
	    }
	    else
		cmd_error("NULL CMD_BUF");
	}
	else
	{
		cmd_error("*** status command ***");
		cmd_error("unknown modifier: ",str1);
//		status = 1;
	}

	return status;
}
