//
//	nsl_convolution_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

// zero-edge vector convolution

nsl_vector NSLconv_zero(nsl_vector& a,nsl_vector& b)
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();

	int simax = saimax + sbimax - 1;

	nsl_vector c(sbimax);
	nsl_vector d(simax);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

	d = d.put_sector(b,saimax/2); 
	for (int i = 0; i < sbimax; i++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
			val = val + av[m] * dv[i+m];
		cv[i] = val;
	   }
	return c;
}
nsl_vector NSLconv_zero(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * b;

	return c;
}
nsl_vector NSLconv_zero(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = a * bv[i];

	return c;
}
nsl_vector NSLconv_zero(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * bv;

	return c;
}
nsl_vector NSLconv_zero(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type av = a.get_data();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = av * bv[i];

	return c;
}
// zero edge matrix convolution 

nsl_matrix NSLconv_zero(nsl_matrix& a,nsl_matrix& b)
{
	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

	nsl_matrix c(sbimax,sbjmax);
	nsl_matrix d(simax,sjmax);

	d = d.put_sector(b,saimax/2,sajmax/2); 

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

	for (int i = 0; i < sbimax; i++)
	   for (int j = 0; j < sbjmax; j++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
	   }

	return c;
}
nsl_matrix NSLconv_zero(nsl_vector& a,nsl_matrix& b) // row,col vector
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);

	num_type* av = a.get_vector();
	num_type** cv = c.get_matrix();
	num_type** dv;

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		d = d.put_sector(b,0,saimax/2); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i][j+m];
			cv[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		d = d.put_sector(b,saimax/2,0); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i+m][j];
			cv[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix NSLconv_zero(nsl_matrix& b,nsl_vector& a) // same as previous
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);

	num_type* av = a.get_vector();
	num_type** cv = c.get_matrix();
	num_type** dv;

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		d = d.put_sector(b,0,saimax/2); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i][j+m];
			cv[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		d = d.put_sector(b,saimax/2,0); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i+m][j];
			cv[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix NSLconv_zero(nsl_matrix& a,nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** av = a.get_matrix();
	num_type bv = b.get_data();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] * bv;

	return c;
}
nsl_matrix NSLconv_zero(nsl_data& a, nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type av = a.get_data();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = av * bv[i][j];

	return c;
}
nsl_matrix NSLconv_zero(nsl_matrix& a,num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] * b;

	return c;
}
nsl_matrix NSLconv_zero(num_type a, nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = a * bv[i][j];

	return c;
}

// wrap around convolution

nsl_vector NSLconv_wrap(nsl_vector& a,nsl_vector& b)
{
      	int saimax = a.get_imax();
      	int sbimax = b.get_imax();

	int sm = saimax/2;

      	int simax = saimax + sbimax;

      	nsl_vector c(sbimax);
      	nsl_vector d(simax);

      	d = d.put_sector(b,saimax/2); 

	int i,m;

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

      	for (i = sm-1; i >= 0; i--)
	   dv[i] = bv[(i-sm+sbimax)%sbimax];
      	for (i = sm+sbimax; i < simax; i++)
	   dv[i] = bv[(i-sm)%sbimax];

      	for (i = 0; i < sbimax; i++)
        {    
              num_type val = (num_type) 0;
              for (m = 0; m < saimax; m++)
                      val = val + av[m] * dv[i+m];
              cv[i] = val;
        }

      return c;
}

