//
//	nsl_data.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// nsl_vector constructors

nsl_data::nsl_data() : (1)
{
	v = 0;
	data_type = 2;
}
nsl_data::nsl_data(nsl_data& a) : (1)
{
	v = a.v;
	data_type = 2;
}

nsl_data::nsl_data(nsl_model* model,char* str) : (str,1)
{
	v = 0;
	data_type = 2;
	model->add_layer(this);
}
nsl_data::nsl_data(nsl_model& model,char* str) : (str,1)
{
	v = 0;
	data_type = 2;
	model.add_layer(this);
}
nsl_data::nsl_data(char* str) : (str,1)
{
	v = 0;
	data_type = 2;
	NSL_MODEL->add_layer(this);
}

nsl_data::nsl_data(char* str,int type) : (str,1) // data type
{
	data_type = type;
	switch (type)
	{
	case 1:
		ival = 0;
		break;
	case 2:
		v = 0;
		break;
	case 3:
		strcpy(sval,"");
		break;
	default:
		data_type = 0;
		cmd_error("*** nsl_data::nsl_data ***");
		cmd_error("Unknown data type: ",type);		
	}
}

// nsl_data destructors

nsl_data::~nsl_data()
{
}

// nsl_data set values

void nsl_data::set_value(int val)
{
	if (able_fg == 0)
		return;

	data_type = 1;
	ival = val;
}
void nsl_data::set_value(float val)
{
	if (able_fg == 0)
		return;

	data_type = 2;
	v = val;
}
void nsl_data::set_value(char* val)
{
	if (able_fg == 0)
		return;

	data_type = 3;
	strcpy(sval,val);
}

// nsl_data assignment

void nsl_data::operator=(nsl_data& a)
{
	if (able_fg == 0)
		return;

	v = a.v;
}
void nsl_data::operator=(double a)
{
	if (able_fg == 0)
		return;

	v = (num_type) a;
}
void nsl_data::operator=(float a)
{
	if (able_fg == 0)
		return;

	v = (num_type) a;
}
void nsl_data::operator=(int a)
{
	if (able_fg == 0)
		return;

	v = (num_type) a;
}

// nsl_data addition

nsl_data operator+(nsl_data& a)
{
	nsl_data c;
	c.v = a.v;
	return c;
}
nsl_data operator+(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	c.v = a.v + b.v;
	return c;
}
nsl_data operator+(nsl_data& a,num_type b)
{
	nsl_data c;
	c.v = a.v + b;
	return c;
}
nsl_data operator+(num_type a, nsl_data& b)
{
	nsl_data c;
	c.v = a + b.v;
	return c;
}

// nsl_data subtruction

nsl_data operator-(nsl_data& a)
{
	nsl_data c;
	c.v = -a.v;
	return c;
}
nsl_data operator-(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	c.v = a.v - b.v;
	return c;
}
nsl_data operator-(nsl_data& a,num_type b)
{
	nsl_data c;
	c.v = a.v - b;
	return c;
}
nsl_data operator-(num_type a, nsl_data& b)
{
	nsl_data c;
	c.v = a - b.v;
	return c;
}

// nsl_data multiplication

nsl_data operator*(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	c.v = a.v * b.v;
	return c;
}
nsl_data operator*(nsl_data& a,num_type b)
{
	nsl_data c;
	c.v = a.v * b;
	return c;
}
nsl_data operator*(num_type a, nsl_data& b)
{
	nsl_data c;
	c.v = a * b.v;
	return c;
}

// nsl_datator pointwise division

nsl_data operator/(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	c.v = a.v / b.v;
	return c;
}
nsl_data operator/(nsl_data& a,num_type b)
{
	nsl_data c;
	c.v = a.v / b;
	return c;
}
nsl_data operator/(num_type a, nsl_data& b)
{
	nsl_data c;
	c.v = a / b.v;
	return c;
}

// stream

extern int nsl_get_str(istream&,char*);

ostream& operator<<(ostream& out,nsl_data& a)
{
	switch (a.data_type)
	{
	case 1:
		out << a.ival << "\n";
		break;
	case 2:
		out << a.v << "\n";
		break;
	case 3:
		out << a.sval << "\n";
		break;
	default:
		cmd_error("*** operator<< ***");
		cmd_error("Unknown data type: ",a.data_type);
	}

	return out;
}
istream& operator>>(istream& in,nsl_data& a)
{
	if (!in)
		cmd_error("Read Error");

//	a.print_status(cout);
//	a.print(cout);
//	cmd_out("Print op>> data: ",a.data_type);

	float vs;
	string str;
	char* aptr[NAME_SIZE];
	char** ptr = aptr;

	// loop until some string is encountered on any line
	while (nsl_get_str(in,str) == 0)
	{
	    if (strcmp(str,"EOF") == 0) // check for EOF
	    {
		cmd_error("No data input value");
		return in;
	    }
	}

	int type = a.get_data_type();

	if (type == 1 || type == 2)
	{
	    vs = strtod(str,ptr); // similar to 'atof(str)' w/ error check

	    if (strcmp(str,*ptr) == 0)
	    {
	    	cmd_error("Bad data input value: ",str);
	    	return in;
	    }
	}

	switch (type)
	{
	case 1:
//		in >> a.ival;
		a.ival = rint(vs);
		break;
	case 2:
//		in >> a.v;
		a.v = vs;
		break;
	case 3:
//		in >> a.sval;
		strcpy(a.sval,str);
		break;
	default:
		cmd_error("*** operator>> ***");
		cmd_error("Unknown data type: ",type);
	}

	return in;
}

// print nsl_data values

void nsl_data::print(ostream& out)
{
	out << "set data_value " << name << "\n" << *this << "\n";
	out.flush();
}

// print status

void nsl_data::print_layer_status(ostream& out)
{
	nsl_layer::print_layer_status(out);

	out.flush();
}
void nsl_data::print_status(ostream& out)
{
	print_layer_status(out);
}

// generic data macro...
nsl_data* get_data_obj(data_type_list* data_list,char* name)
{
	if (data_list != NULL)
	{
	    data_type_iterator next(*data_list);
	    nsl_data* data;
	    while (data = next())
		if (strcmp(name,data->get_name()) == 0)
			return data;
	}
	else
	{
//	    cmd_error("*** get_data_obj ***");
//	    cmd_error("NULL data_list");
	    return (nsl_data*) 0;
	}

//	cmd_error("*** get_data_obj ***");
//	cmd_error("Unknown data name: ",name);

	return (nsl_data*) 0;
}
nsl_data* get_data_obj(data_type_list* data_list,int index)
{
	if (data_list != NULL)
	{
	    data_type_iterator next(*data_list);
	    nsl_data* data;
	    while (data = next())
		if (index == data->get_index())
			return data;
	}
	else
	{
//	    cmd_error("*** get_data_obj ***");
//	    cmd_error("NULL data_list");
	    return (nsl_data*) 0;
	}

//	cmd_error("*** get_data_obj ***");
//	cmd_error("Unknown data index: ",index);

	return (nsl_data*) 0;
}
