//
//	nsl_diff.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

nsl_diff::nsl_diff()
{
	type = 1; // 1 - EULER, 2 - INTERPOLATION
}
nsl_diff::~nsl_diff()
{
}

// perform DIFF computation

void nsl_diff::operator=(num_type a)
{
	if (type == 2) // INTERPOLATION
	{
	    num_type p = exp(-dt/tm);
	    if (layer_type == 1)
		*((nsl_data*) layer) = *((nsl_data*) layer) + (1-p)*a;
	    else if (layer_type == 2)
		*((nsl_vector*) layer) = *((nsl_vector*) layer) + (1-p)*a;	
	    else if (layer_type == 3)
		*((nsl_matrix*) layer) = *((nsl_matrix*) layer) + (1-p)*a;
	}
	else // if (type == 1) // EULER
	{
	    if (layer_type == 1)
		*((nsl_data*) layer) = *((nsl_data*) layer) + dt/tm*a;
	    else if (layer_type == 2)
		*((nsl_vector*) layer) = *((nsl_vector*) layer) + dt/tm*a;	
	    else if (layer_type == 3)
		*((nsl_matrix*) layer) = *((nsl_matrix*) layer) + dt/tm*a;
	}
}
void nsl_diff::operator=(nsl_data& a)
{
	if (type == 2) // INTERPOLATION
	{
	    num_type p = exp(-dt/tm);
	    if (layer_type == 1)
		*((nsl_data*) layer) = *((nsl_data*) layer) + (1-p)*a;
	    else if (layer_type == 2)
		*((nsl_vector*) layer) = *((nsl_vector*) layer) + (1-p)*a;	
	    else if (layer_type == 3)
		*((nsl_matrix*) layer) = *((nsl_matrix*) layer) + (1-p)*a;
	}
	else // if (type == 1) // EULER
	{
	    if (layer_type == 1)
		*((nsl_data*) layer) = *((nsl_data*) layer) + dt/tm*a;
	    else if (layer_type == 2)
		*((nsl_vector*) layer) = *((nsl_vector*) layer) + dt/tm*a;
	    else if (layer_type == 3)
		*((nsl_matrix*) layer) = *((nsl_matrix*) layer) + dt/tm*a;
	}	
}
void nsl_diff::operator=(nsl_vector& a)
{
	if (type == 2) // INTERPOLATION
	{
	    num_type p = exp(-dt/tm);
	    if (layer_type == 1)
		cmd_out("DIFF: Bad assignment vector to data");
	    else if (layer_type == 2)
		*((nsl_vector*) layer) = *((nsl_vector*) layer) + (1-p)*a;	
	    else if (layer_type == 3)
		cmd_out("DIFF: Bad assignment vector to matrix");
	}
	else // if (type == 1) // EULER
	{
	    if (layer_type == 1)
		cmd_out("DIFF: Bad assignment vector to data");
	    else if (layer_type == 2)
		*((nsl_vector*) layer) = *((nsl_vector*) layer) + dt/tm*a;
	    else if (layer_type == 3)
		cmd_out("DIFF: Bad assignment vector to matrix");
	}
}
void nsl_diff::operator=(nsl_matrix& a)
{
	if (type == 2) // INTERPOLATION
	{
	    num_type p = exp(-dt/tm);
	    if (layer_type == 1)
		cmd_out("DIFF: Bad assignment matrix to data");
	    else if (layer_type == 2)
		cmd_out("DIFF: Bad assignment matrix to vector");
	    else if (layer_type == 3)
		*((nsl_matrix*) layer) = *((nsl_matrix*) layer) + (1-p)*a;
	}
	else // if (type == 1) // EULER
	{
	    if (layer_type == 1)
		cmd_out("DIFF: Bad assignment matrix to data");
	    else if (layer_type == 2)
		cmd_out("DIFF: Bad assignment matrix to vector");
	    else if (layer_type == 3)
		*((nsl_matrix*) layer) = *((nsl_matrix*) layer) + dt/tm*a;
	}
}

// set DIFF parameters 

nsl_diff& nsl_diff::eq(nsl_data& m,num_type k)
{
	dt = NSL_MODEL->get_delta();
	tm = k;
	layer = &m;
	layer_type = 1;

	return *this;	
}
nsl_diff& nsl_diff::eq(nsl_vector& m,num_type k)
{
	dt = NSL_MODEL->get_delta();
	tm = k;
	layer = &m;
	layer_type = 2;

	return *this;	
}
nsl_diff& nsl_diff::eq(nsl_matrix& m,num_type k)
{
	dt = NSL_MODEL->get_delta();
	tm = k;
	layer = &m;
	layer_type = 3;

	return *this;	
}

nsl_diff& nsl_diff::eq(nsl_data& m,nsl_data& k)
{
	dt = NSL_MODEL->get_delta();
	tm = k.elem();
	layer = &m;
	layer_type = 1;

	return *this;	
}
nsl_diff& nsl_diff::eq(nsl_vector& m,nsl_data& k)
{
	dt = NSL_MODEL->get_delta();
	tm = k.elem();
	layer = &m;
	layer_type = 2;

	return *this;	
}
nsl_diff& nsl_diff::eq(nsl_matrix& m,nsl_data& k)
{
	dt = NSL_MODEL->get_delta();
	tm = k.elem();
	layer = &m;
	layer_type = 3;

	return *this;	
}

