/**************************************************************/
/*                                                            */
/*                       nsl_file.c 		              */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

//static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;
extern nsl_interpreter NSL;

nsl_file::nsl_file(char* fullstr)
{
	char *str,*fname0;

	int len = strlen(fullstr);

      	if ((str = (char *) strrchr(fullstr,'/')) != NULL)
	{
		fname0 = str + 1; // pointer 1 after sname
		int len0 = strlen(str);
		strncpy(dname,fullstr,len-len0);
	}
	else
	{
		fname0 = fullstr;
		strcpy(dname,".");
	}

      	if ((str = (char *) strchr(fname0,'.')) != NULL)
	{
	    if (strncmp(str,".nsl",4) == 0)
	    {
		strcpy(fname,fname0);
		strncpy(name,fname0,strlen(fname0)-strlen(str));
	    }
	    else
	    {
		cmd_error("Not a '.nsl' file: ",fname0);
		return; // -1;
	    }
	}
	else
	{
	    sprintf(fname,"%s.nsl",fname0);
	    strcpy(name,fname0);
	}

	sprintf(fullname,"%s/%s",dname,fname);
}
nsl_file::nsl_file()
{
	strcpy(dname,".");
	strcpy(name,"TMP");
	strcpy(fname,"TMP.nsl.log");
	sprintf(fullname,"%s/%s",dname,fname);
}
nsl_file::~nsl_file()
{
}
void nsl_file::set_fullname()
{
	string str;
	strcpy(str,"");

	switch (type)
	{
	case 0: // strcpy(str,".nsl");
		break;
	case 1: strcpy(str,".log");
		break;
	case 2: strcpy(str,".dat");
		break;
	case 3: strcpy(str,".ras");
		break;
	case 4: strcpy(str,".ps");
		break;
	default:
		cmd_error("Unknown file type: ",type);
	}
	sprintf(fullname,"%s/%s.nsl%s.%d",dname,name,str,version_num);
}
void nsl_file::print_status(ostream& out)
{
	out << "\n";
	out << "// dir name: " << dname << "\n";
//	out << "// name: " << name << "\n";
	out << "// file name: " << fname << "\n";
//	out << "// fullname: " << fullname << "\n";
//	out << "// file index: " << index << "\n";
	if (io_type == 0)
		out << "// type: INPUT\n";
	else if (io_type == 1)
		out << "// type: OUTPUT\n";
	else
		out << "// unknown type: " << io_type << "\n";
/*	switch (type)
	{
	case 0: out << "// format: nsl\n";
		break;
	case 1: out << "// format: log\n";
		break;
	case 2: out << "// format: dat\n";
		break;
	case 3: out << "// format: ras\n";
		break;
	case 4: out << "// format: ps\n";
		break;
	default:
		out << "// format: " << type << "\n";
	}
*/	if (able_fg == 1)
		out << "// open\n"; // enabled
	else
		out << "// closed\n"; // disabled

	out.flush();
}
int nsl_file::load()
{
	return read();
}
int nsl_file::read()
{
	status = 1;

	if (!file)
	{
		cmd_error("Cannot read file: ",fullname);
		return -1;
	}

        while (file && file->peek() != EOF && status == 1)
	{
		NSL << (*file);
		status = NSL.get_status();
	}

	if (status == -1)
		cmd_error("Unknown Model");

	return status;
}
int nsl_file::write()
{
	status = 1;

	switch (file->rdstate())
	{
	case 0: // _good
		able_fg = 1;
		status = 1;
		break;
	case 1: // _eof
		break;
	case 2: // _fail
		cmd_error("Cannot write file: ",fullname);
		able_fg = 0;
		status = -1;
		break;
	case 3: // _bad
		break;
	default:
		cmd_error("Unknown open file state");
	}

	return status;
}
int nsl_file::open(int i)
{
	set_io_type(i);
	status = open();

	return status;
}
int nsl_file::open()
{
	file = new fstream;

	status = 1;

	if (file == NULL)
	{
		cmd_out("// Bad file pointer");
		return -1;
	}
	
	if (io_type == 0)
	{
		file->open(fullname,input);
//		cmd_out("// Opening input file: ",fullname);
	}
	else if (io_type == 1)
	{
		file->open(fullname,output);
//		cmd_out("// Opening output file: ",fullname);
	}
	else
	{
		cmd_error("Unknown file io_type: ",io_type);
	}

	switch (file->rdstate())
	{
	case 0: // _good
		able_fg = 1;
		status = 1;
		break;
	case 1: // _eof
		break;
	case 2: // _fail
		cmd_error("Cannot open file: ",fullname);
		able_fg = 0;
		status = -1;
		break;
	case 3: // _bad
		break;
	default:
		cmd_error("Unknown open file state");
	}
//	able_fg = 1;

	return status;
}
int nsl_file::close()
{
	file->close();

	if (io_type == 0)
	{
//		cmd_out("// Closing read file: ",fullname);
	}
	else if (io_type == 1)
	{
//		cmd_out("// Closing write file: ",fullname);
	}
	else
	{
		cmd_error("Unknown file io_type: ",io_type);
	}
	able_fg = 0;

	return 1;
}
int nsl_file::attach_stdout()
{
	file->attach(1); // standard output

	return 1;
}
int nsl_file::attach_stdin()
{
	file->attach(0); // standard input

	return 1;
}

