//
//	nsl_matrix_add_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

// matrix addition

nsl_matrix NSLadd(nsl_matrix& a)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** cv = c.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j];

	return c;
}
nsl_matrix NSLadd(nsl_matrix& a,nsl_matrix& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("*** nsl_matrix::,NSLadd ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error_exit("Inconsistent matrix sizes");
	}

	nsl_matrix c(aimax,ajmax);

	num_type** cv = c.get_matrix();
	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] + bv[i][j];

	return c;
}
nsl_matrix NSLadd(nsl_matrix& a,num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** cv = c.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] + b;

	return c;
}
nsl_matrix NSLadd(num_type a, nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type** cv = c.get_matrix();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = a + bv[i][j];

	return c;
}
nsl_matrix NSLadd(nsl_matrix& a,nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** cv = c.get_matrix();
	num_type** av = a.get_matrix();
	num_type bv = b.get_data();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] + bv;

	return c;
}
nsl_matrix NSLadd(nsl_data& a, nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type** cv = c.get_matrix();
	num_type** bv = b.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = av + bv[i][j];

	return c;
}

