//
//	nsl_misc_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

// class expansion

nsl_matrix NSLvec_to_mat(nsl_matrix& ref,nsl_vector& a)
{
	int vec_type = a.get_vec_type();

	int simax = ref.get_imax();
	int sjmax = ref.get_jmax();

	int s = a.get_imax();

	nsl_matrix c(simax,sjmax);
	
	if (vec_type == 0) // row vector
	{
	    if (s != sjmax)
	    {
		cmd_error("*** NSLvec_to_mat ***");
		cmd_error("vector jmax: ",s);
		cmd_error("matrix jmax: ",sjmax);
		cmd_error_exit("Bad vector size");
	    }

	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    c.v[i][j] = a.v[j];
	}
	if (vec_type == 1) // col vector
	{
	    if (s != simax)
	    {
		cmd_error("*** NSLvec_to_mat ***");
		cmd_error("vector imax: ",s);
		cmd_error("matrix imax: ",simax);
		cmd_error_exit("Bad vector size");
	    }

	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    c.v[i][j] = a.v[i];
	}

	return c;
}

nsl_vector NSLmat_to_vec(nsl_vector& ref,nsl_matrix& a)
{
	int vec_type = ref.get_vec_type();

	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	int s = ref.get_imax();

	nsl_vector c(s);
	
	if (vec_type == 0) // row vector
	{
	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    c.v[j] = a.v[i][j] + c.v[j];
	}
	if (vec_type == 1) // col vector
	{
	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    c.v[i] = a.v[i][j] + c.v[i];
	}

	return c;
}

nsl_matrix NSLrow_vec_to_mat(nsl_matrix& ref,nsl_vector& a)
{
	int simax = ref.get_imax();
	int sjmax = ref.get_jmax();

	int s = a.get_imax();

	if (s != sjmax)
	{
		cmd_error("*** NSLrow_vec_to_mat ***");
		cmd_error("vector jmax: ",s);
		cmd_error("matrix jmax: ",sjmax);
		cmd_error_exit("Bad vector size");
	}

	nsl_matrix c(simax,sjmax);
	
	for (int i = 0; i < simax; i++)
	    for (int j = 0; j < sjmax; j++)
		c.v[i][j] = a.v[j];
	return c;
}

nsl_matrix NSLcol_vec_to_mat(nsl_matrix& ref,nsl_vector& a)
{ 
	int simax = ref.get_imax();
	int sjmax = ref.get_jmax();

	int s = a.get_imax();

	if (s != simax)
	{
		cmd_error("*** NSLcol_vec_to_mat ***");
		cmd_error("vector imax: ",s);
		cmd_error("matrix imax: ",simax);
		cmd_error_exit("Bad vector size");
	}

	nsl_matrix c(simax,sjmax);
	
	for (int i = 0; i < simax; i++)
	    for (int j = 0; j < sjmax; j++)
		c.v[i][j] = a.v[i];
	return c; 
}

nsl_vector NSLmat_to_row_vec(nsl_vector& ref,nsl_matrix& a) // ref not used
{
	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	nsl_vector c(sjmax);
		
	for (int i = 0; i < simax; i++)
	   for (int j = 0; j < sjmax; j++)
		c.v[j] = a.v[i][j] + c.v[j];
	return c;
}
nsl_vector NSLmat_to_col_vec(nsl_vector& ref,nsl_matrix& a) // ref not used
{
	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	nsl_vector c(simax);
	
	for (int j = 0; j < sjmax; j++)
	    for (int i = 0; i < simax; i++)
		c.v[i] = a.v[i][j] + c.v[i];

	return c;
}

// update as of 12/1/90

nsl_matrix NSLrow_vec_to_mat(nsl_vector& a,int simax)
{
	int sjmax = a.get_imax();

	nsl_matrix c(simax,sjmax);
	
	for (int i = 0; i < simax; i++)
	    for (int j = 0; j < sjmax; j++)
		c.v[i][j] = a.v[j];
	return c;
}

nsl_matrix NSLcol_vec_to_mat(nsl_vector& a,int sjmax)
{
	int simax = a.get_imax();

	nsl_matrix c(simax,sjmax);
	
	for (int i = 0; i < simax; i++)
	    for (int j = 0; j < sjmax; j++)
		c.v[i][j] = a.v[i];
	return c;
}
nsl_vector NSLmat_to_row_vec(nsl_matrix& a)
{
	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	nsl_vector c(sjmax);
		
	for (int i = 0; i < simax; i++)
	   for (int j = 0; j < sjmax; j++)
		c.v[j] = a.v[i][j] + c.v[j];
	return c;
}
nsl_vector NSLmat_to_col_vec(nsl_matrix& a)
{
	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	nsl_vector c(simax);
	
	for (int j = 0; j < sjmax; j++)
	    for (int i = 0; i < simax; i++)
		c.v[i] = a.v[i][j] + c.v[i];
	return c;
}

