//
//	nsl_stim_data.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// nsl_stim_data

nsl_stim_data::nsl_stim_data() : ()
{
	input_data = 0;
}
nsl_stim_data::nsl_stim_data(nsl_input_data* dat) : ()
{
	input_data = dat;
}
nsl_stim_data::~nsl_stim_data()
{
}

void nsl_stim_data::proc()
{
}

void nsl_stim_data::run()
{
	nsl_input_data* Q = input_data;

	float dt = NSL_MODEL->get_delta();
	float ts = NSL_MODEL->get_time();
	int its = NSL_MODEL->get_time_step();

	if (mode == 0)
		proc(); // virtual
	else if (time_list != NULL)
	{
	   nsl_time_iterator next(*time_list);
	   nsl_time_obj* time_obj;
	   while (time_obj = next())
	   {
	    	if (time_obj->get_able_fg() == 0)
			continue;

		float t0 = time_obj->get_t0();
		float t1 = time_obj->get_t1();

		if ((ts >= t0 && ts <= t1))
		{
			proc(); // virtual
		}
	   }
	}
	else
	   cmd_out("NULL 'time_list'");
}
void nsl_stim_data::print(ostream& out)
{
	nsl_stim_obj::print(out);
}
void nsl_stim_data::print_stim(ostream& out)
{
	print(out);
}

// nsl_block_data

nsl_block_data::nsl_block_data() : ()
{
	val = 1.0;
}
nsl_block_data::nsl_block_data(nsl_input_data* dat) : (dat)
{
	val = 1.0;
}
nsl_block_data::~nsl_block_data()
{
}

void nsl_block_data::proc()
{
	nsl_input_data* Q = input_data;

	float k = (Q->get_overlapfg() == 1) ? 1.0 : 0.0;

        Q->elem() = k*Q->elem() + val;
}
void nsl_block_data::print(ostream& out)
{
	nsl_stim_data::print(out);

	out << "set stim_val " << val << "\n";
	out.flush();
}
void nsl_block_data::print_stim(ostream& out)
{
	print(out);
}

// nsl_icon_data

nsl_icon_data::nsl_icon_data() : ()
{
	icon = 0;
}
nsl_icon_data::nsl_icon_data(nsl_input_data* dat) : (dat)
{
	icon = 0;
}
nsl_icon_data::~nsl_icon_data()
{
}

void nsl_icon_data::proc()
{
	nsl_input_data* Q = input_data;

	float k = (Q->get_overlapfg() == 1) ? 1.0 : 0.0;

        Q->elem() = k*Q->elem() + icon->elem();
}

void nsl_icon_data::print(ostream& out)
{
	nsl_stim_data::print(out);

	if (icon != NULL)
		icon->print(out);
	else
		cmd_out("NULL icon");
}
void nsl_icon_data::print_stim(ostream& out)
{
	print(out);
}
