//
//	nsl_stim_matrix.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// nsl_stim_matrix

nsl_stim_matrix::nsl_stim_matrix() : ()
{
	input_matrix = 0;

	type = 0;

	x0 = 0;
	y0 = 0;
	tx0 = 0;
	ty0 = 0;
	xc = 0;
	yc = 0;
	txc = 0;
	tyc = 0;

	vx = 0.0;
	vy = 0.0;
}
nsl_stim_matrix::nsl_stim_matrix(nsl_input_matrix* mat) : ()
{
	input_matrix = mat;

	type = 0;

	x0 = 0;
	y0 = 0;
	tx0 = 0;
	ty0 = 0;
	xc = 0;
	yc = 0;
	txc = 0;
	tyc = 0;

	vx = 0.0;
	vy = 0.0;
}
nsl_stim_matrix::~nsl_stim_matrix()
{
}

void nsl_stim_matrix::proc()
{
}

void nsl_stim_matrix::run()
{
	nsl_input_matrix* Q = input_matrix;

	float dt = NSL_MODEL->get_delta();
	float ts = NSL_MODEL->get_time();
	int its = NSL_MODEL->get_time_step();

	int dt0 = step;		// recalculation frequency
	if (dt0 == 0 && ts == 0.0)
	{
		txc = xc;
		tyc = yc;
		tx0 = x0;
		ty0 = y0;
	}
	else if (dt0 != 0 && its%step == 0)
	{
		txc = xc + ts*vx;
		tyc = yc + ts*vy;
		tx0 = x0 + ts*vx;
		ty0 = y0 + ts*vy;
	}

	if (mode == 0)
		proc(); // virtual
	else if (time_list != NULL)
	{
	   nsl_time_iterator next(*time_list);
	   nsl_time_obj* time_obj;
	   while (time_obj = next())
	   {
	    	if (time_obj->get_able_fg() == 0)
			continue;

		float t0 = time_obj->get_t0();
		float t1 = time_obj->get_t1();

		if ((ts >= t0 && ts <= t1))
		{
			proc(); // virtual
		}
	   }
	}
	else
	   cmd_out("NULL 'time_list'");
}
void nsl_stim_matrix::print(ostream& out)
{
	nsl_stim_obj::print(out);

	if (type == 1) 
	   out << "set stim_ref CORNER\n";
	else // if (type == 0)
	   out << "set stim_ref CENTER\n";
	out << "set stim_x0 " << x0 << "\n";
	out << "set stim_y0 " << y0 << "\n";
	out << "// stim_tx0: " << tx0 << "\n";
	out << "// stim_ty0: " << ty0 << "\n";
	out << "set stim_xc " << xc << "\n";
	out << "set stim_yc " << yc << "\n";
	out << "// stim_txc: " << txc << "\n";
	out << "// stim_tyc: " << tyc << "\n";
	out << "set stim_vx " << vx << "\n";
	out << "set stim_vy " << vy << "\n";
}
void nsl_stim_matrix::print_stim(ostream& out)
{
	print(out);
}

// nsl_block_matrix

nsl_block_matrix::nsl_block_matrix() : ()
{
	val = 1.0;
 	dx = 0.0;
 	dy = 0.0;
}
nsl_block_matrix::nsl_block_matrix(nsl_input_matrix* mat) : (mat)
{
	val = 1.0;
 	dx = 0.0;
 	dy = 0.0;
}
nsl_block_matrix::~nsl_block_matrix()
{
}

void nsl_block_matrix::proc()
{
 	int i,j,a,b;
	int qi0,qj0,qic,qjc;

	nsl_input_matrix* Q = input_matrix;

	int xz = Q->get_xz(); // zero x element
	int yz = Q->get_yz(); // zero y element
	float mdx = Q->get_dx(); // x interval 
	float mdy = Q->get_dy(); // y interval

	int idx = (int)(dx/mdx+0.5); // int width
	int idy = (int)(dy/mdy+0.5); // int height

	idx = (idx >= 1) ? idx-1 : 0; // >= 0
	idy = (idy >= 1) ? idy-1 : 0; // >= 0

	int qimax = Q->get_imax();
	int qjmax = Q->get_jmax();

	if (type == 0) // reference xc,yc
	{
		qjc = (int)(txc/mdx+0.5) + xz;  // int left corner
		qic = (int)(tyc/mdy+0.5) + yz;  // int top corner
		qj0 = qjc - idx/2;
		qi0 = qic - idy/2;
	}
	else // reference x0,y0
	{
		qj0 = (int)(tx0/mdx+0.5) + xz;  // int left corner
		qi0 = (int)(ty0/mdy+0.5) + yz;  // int top corner
	}

	int qj1 = qj0 + idx;
	int qi1 = qi0 + idy;

	float k = (Q->get_overlapfg() == 1) ? 1.0 : 0.0;

        for (i = qi0; i <= qi1; i++)
            for (j = qj0; j <= qj1; j++)
            {
//               a = i - qi0;
//               b = j - qj0;
                a = i;
                b = j;
                if (0 <= a && a < qimax && 0 <= b && b < qjmax)
                   Q->elem(a,b) = k*Q->elem(a,b) + val;
	    }
}

void nsl_block_matrix::print(ostream& out)
{
	nsl_stim_matrix::print(out);

	out << "set stim_val " << val << "\n";
	out << "set stim_dx " << dx << "\n";
	out << "set stim_dy " << dy << "\n";
	out.flush();
}
void nsl_block_matrix::print_stim(ostream& out)
{
	print(out);
}

// nsl_icon_matrix

nsl_icon_matrix::nsl_icon_matrix() : ()
{
	icon = 0;
}
nsl_icon_matrix::nsl_icon_matrix(nsl_input_matrix* mat) : (mat)
{
	icon = 0;
}
nsl_icon_matrix::~nsl_icon_matrix()
{
}

void nsl_icon_matrix::proc()
{
        int i,j,a,b;

	nsl_input_matrix* Q = input_matrix;

	int xz = Q->get_xz(); 	// axis x element center
	int yz = Q->get_yz(); 	// axis y element center
	float mdx = Q->get_dx(); // distance between horizontal (x) elements
	float mdy = Q->get_dy(); // distance between vertical (y) elements

	int qimax = Q->get_imax();
	int qjmax = Q->get_jmax();

	int imax = icon->get_imax();
	int jmax = icon->get_jmax();

	int qj0 = (int)(tx0/mdx+0.5) + xz;  // int left corner // ??
	int qi0 = (int)(ty0/mdy+0.5) + yz;  // int top corner // ??

	int qi1 = qi0 + imax;
	int qj1 = qj0 + jmax;

	float k = (Q->get_overlapfg() == 1) ? 1.0 : 0.0;

        for (i = qi0; i <= qi1; i++)
            for (j = qj0; j <= qj1; j++)
            {
                a = i - qi0;
                b = j - qj0;
                if (0 <= a && a < qimax && 0 <= b && b < qjmax)
                    Q->elem(a,b) = k*Q->elem(a,b) + icon->elem(i,j);
            }
}

void nsl_icon_matrix::print(ostream& out)
{
	nsl_stim_matrix::print(out);

	if (icon != NULL)
		icon->print(out);
	else
		cmd_out("NULL icon");
}
void nsl_icon_matrix::print_stim(ostream& out)
{
	print(out);
}
