//
//	nsl_stim_vector.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// nsl_stim_vector

nsl_stim_vector::nsl_stim_vector() : ()
{
	input_vector = 0;

	type = 0;

	x0 = 0;
	tx0 = 0;
	xc = 0;
	txc = 0;

	vx = 0.0;
}
nsl_stim_vector::nsl_stim_vector(nsl_input_vector* vec) : ()
{
	input_vector = vec;

	type = 0;

	x0 = 0;
	tx0 = 0;
	xc = 0;
	txc = 0;

	vx = 0.0;
}
nsl_stim_vector::~nsl_stim_vector()
{
}

void nsl_stim_vector::proc()
{
}

void nsl_stim_vector::run()
{
	nsl_input_vector* Q = input_vector;

	float dt = NSL_MODEL->get_delta();
	float ts = NSL_MODEL->get_time();
	int its = NSL_MODEL->get_time_step();

	int dt0 = step;		// recalculation frequency
	if (dt0 == 0 && ts == 0.0)
	{
		txc = xc;
		tx0 = x0;
	}
	else if (dt0 != 0 && its%step == 0)
	{
		txc = xc + ts*vx;
		tx0 = x0 + ts*vx;
	}

	if (mode == 0)
		proc(); // virtual

	else if (time_list != NULL)
	{
	   nsl_time_iterator next(*time_list);
	   nsl_time_obj* time_obj;
	   while (time_obj = next())
	   {
	    	if (time_obj->get_able_fg() == 0)
			continue;

		float t0 = time_obj->get_t0();
		float t1 = time_obj->get_t1();

		if ((ts >= t0 && ts <= t1))
		{
			proc(); // virtual
		}
	   }
	}
	else
	   cmd_out("NULL 'time_list'");
}
void nsl_stim_vector::print(ostream& out)
{
	nsl_stim_obj::print(out);

	if (type == 1) 
	   out << "set stim_ref CORNER\n";
	else // if (type == 0)
	   out << "set stim_ref CENTER\n";
	out << "set stim_x0 " << x0 << "\n";
	out << "// tx0: " << tx0 << "\n";
	out << "set stim_xc " << xc << "\n";
	out << "// txc: " << txc << "\n";
	out << "set stim_vx " << vx << "\n";
}
void nsl_stim_vector::print_stim(ostream& out)
{
	print(out);
}

// nsl_block_vector

nsl_block_vector::nsl_block_vector() : ()
{
	val = 1.0;
 	dx = 0.0;
}
nsl_block_vector::nsl_block_vector(nsl_input_vector* mat) : (mat)
{
	val = 1.0;
 	dx = 0.0;
}
nsl_block_vector::~nsl_block_vector()
{
}

void nsl_block_vector::proc()
{
 	int i,a;
	int qi0,qic;

	nsl_input_vector* Q = input_vector;

	int xz = Q->get_xz(); // zero x element
	float mdx = Q->get_dx(); // x interval 

	int idx = (int)(dx/mdx+0.5); // int width

	idx = (idx >= 1) ? idx-1 : 0; // >= 0

	int qimax = Q->get_imax();

	if (type == 0) // reference xc
	{
		qic = (int)(txc/mdx+0.5) + xz;  // int left corner
		qi0 = qic - idx/2;
	}
	else // reference x0
	{
		qi0 = (int)(tx0/mdx+0.5) + xz;  // int left corner
	}

	int qi1 = qi0 + idx;

	float k = (Q->get_overlapfg() == 1) ? 1.0 : 0.0;

        for (i = qi0; i <= qi1; i++)
    	{
//              a = i - qi0;
                a = i;
                if (0 <= a && a < qimax)
                   Q->elem(a) = k*Q->elem(a) + val;
	}
}
void nsl_block_vector::print(ostream& out)
{
	nsl_stim_vector::print(out);

	out << "set stim_val " << val << "\n";
	out << "set stim_dx " << dx << "\n";
	out.flush();
}
void nsl_block_vector::print_stim(ostream& out)
{
	print(out);
}


// nsl_icon_vector

nsl_icon_vector::nsl_icon_vector() : ()
{
	icon = 0;
}
nsl_icon_vector::nsl_icon_vector(nsl_input_vector* vec) : (vec)
{
	icon = 0;
}
nsl_icon_vector::~nsl_icon_vector()
{
}

void nsl_icon_vector::proc()
{
        int i,a;

	nsl_input_vector* Q = input_vector;

	int xz = Q->get_xz(); 	// axis x element center
	float mdx = Q->get_dx(); // distance between horizontal (x) elements

	int qimax = Q->get_imax();

	int imax = icon->get_imax();

	int qi0 = (int)(tx0/mdx+0.5) + xz;  // int left corner // ??

	int qi1 = qi0 + imax;

	float k = (Q->get_overlapfg() == 1) ? 1.0 : 0.0;

        for (i = qi0; i <= qi1; i++)
        {
                a = i - qi0;
                if (0 <= a && a < qimax)
                    Q->elem(a) = k*Q->elem(a) + icon->elem(i);
        }
}

void nsl_icon_vector::print(ostream& out)
{
	nsl_stim_vector::print(out);

	if (icon != NULL)
		icon->print(out);
	else
		cmd_out("NULL icon");
}
void nsl_icon_vector::print_stim(ostream& out)
{
	print(out);
}
