//
//	nsl_system.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"
# include "nsl_system.h"

extern nsl_model* NSL_MODEL;
extern window_interface* NSL_INTERFACE;

nsl_system* NSL_SYSTEM;

nsl_system::nsl_system()
{
	NSL_SYSTEM = this;

	model_total = 0;
	nsl_model_list = new model_type_list;

	file_total = 0;
	nsl_file_list = new file_type_list;

	log_total = 0;
	nsl_log_list = new file_type_list;

	interface = (window_interface*) 0;

	nsl_graph_list = new data_type_list; 
	graph_total = 0;

	init_graph_list(); // different library graphs
}

// add model
 
int nsl_system::add_model(nsl_model* model)
{
	model->set_index(++model_total);
	nsl_model_list->append(model);

	return model_total;
}

// get model

nsl_model* nsl_system::get_model(char* name)
{
	char* str;
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
		str = model->get_name();
		if (strcmp(name,str) == 0)
			return model;
	}
	return (nsl_model*) 0;
}
nsl_model* nsl_system::get_model(int index)
{
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
		if (index == model->get_index())
			return model;
	}
	return (nsl_model*) 0;
}

//enable, disable model

void nsl_system::enable_all_model()
{
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
		model->enable();
}
void nsl_system::enable_model(char* name)
{
	char* str;
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
		str = model->get_name();
		if (strcmp(name,str) == 0)
			model->enable();
//		else // can only be a sinlge enabled model
//			model->disable();
	}
}
void nsl_system::enable_model(int index)
{
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
		if (index == model->get_index())
			model->enable();
//		else // can only be a sinlge enabled model
//			model->disable();
	}
}
void nsl_system::disable_all_model()
{
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
		model->disable();
}
void nsl_system::disable_model(char* name)
{
	char* str;
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
		str = model->get_name();
		if (strcmp(name,str) == 0)
			model->disable();
	}
}
void nsl_system::disable_model(int index)
{
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
		if (index == model->get_index())
			model->disable();
	}
}

// print model info

void nsl_system::print_status(ostream& out)
{
	out << "\n// total models: " << model_total << "\n";
	out.flush();
}

void nsl_system::print_interface_status(ostream& out)
{
	interface->print_status(out);
}
void nsl_system::print_model_status(ostream& out)
{
	print_status(out);

	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	{
	    out << "// " << model->get_index() << ": ";
	    out << model->get_name() << "\t";
	    if (model->get_able_fg() == 1)
	   	out << "(enabled)\n";
	    else // if (model->get_able_fg() == 0)
	   	out << "(disabled)\n";
	    out.flush();
	}
}
void nsl_system::print_all_model_status(ostream& out)
{
	model_type_iterator next(*nsl_model_list);
	nsl_model* model;
	while (model = next())
	    model->print_status(out);
}
void nsl_system::print_model_status(ostream& out,char* str)
{
	nsl_model* model;
	if ((model = get_model(str)) != NULL)
	    model->print_status(out);
	else
	    cmd_error("Unknown Model: ",str);
}
void nsl_system::print_model_status(ostream& out,int i)
{
	nsl_model* model;
	if ((model = get_model(i)) != NULL)
	    model->print_status(out);
	else
	    cmd_error("Unknown Model Index: ",i);
}

// i/o files


int nsl_system::add_file(nsl_file* sfile)
{
	sfile->set_index(++file_total);
	nsl_file_list->append(sfile);

	return file_total;
}
nsl_file* nsl_system::get_file(char* name) // any dir
{
	char* str;
	file_type_iterator next(*nsl_file_list);
	nsl_file* sfile;
	while (sfile = next())
	{
		str = sfile->get_fname();	// w/o file ending
		if (strcmp(name,str) == 0)
			return sfile;
		str = sfile->get_name(); 	// w/ file ending
		if (strcmp(name,str) == 0)
			return sfile;
	}
	return (nsl_file*) 0;
}
nsl_file* nsl_system::get_file(int index)
{
	file_type_iterator next(*nsl_file_list);
	nsl_file* sfile;
	while (sfile = next())
	{
		if (index == sfile->get_index())
			return sfile;
	}
	return (nsl_file*) 0;
}
void nsl_system::print_all_file_status(ostream& out)
{
	file_type_iterator next(*nsl_file_list);
	nsl_file* sfile;
	out << "// total active files: " << file_total << "\n";
	while (sfile = next())
		sfile->print_status(out);
	out.flush();
}
void nsl_system::print_file_status(ostream& out,char* str)
{
	nsl_file* sfile;
	if ((sfile = get_file(str)) != NULL)
		sfile->print_status(out);
	else
		cmd_error("Unknown File Index");
}
void nsl_system::print_file_status(ostream& out,int index)
{
	nsl_file* sfile;
	if ((sfile = get_file(index)) != NULL)
		sfile->print_status(out);
	else
		cmd_error("Unknown File Index");
}

// log file stack

int nsl_system::push_log_file(nsl_file* sfile)
{
	nsl_log_list->insert(sfile);

	return ++log_total;
}
nsl_file* nsl_system::pop_log_file()
{
	nsl_file* sfile = nsl_log_list->get();
	if (sfile != NULL)
		--log_total;
	else
		cmd_error("Empty log file stack");

	return sfile;
}
void nsl_system::print_log_file_status(ostream& out)
{
	print_status(out);

	file_type_iterator next(*nsl_log_list);
	nsl_file* sfile;
	while (sfile = next())
		sfile->print_status(out);
}

// graphs

int nsl_system::set_graph(char* name)
{
	nsl_data* graph = new nsl_data;
	graph->set_name(name);

	return add_graph(graph);
}
int nsl_system::add_graph(nsl_data* graph)
{
	graph->set_index(++graph_total);
	nsl_graph_list->append(graph);

	return graph_total;
}
nsl_data* nsl_system::get_graph(char* name)
{
        if (nsl_graph_list != NULL)
        {
           char* str;
           data_type_iterator next(*nsl_graph_list);
           nsl_data* graph;
           while (graph = next())
           {
                str = graph->get_name();
		if (strcmp(name,str) == 0)
			return graph;
	   }
        }
        else
           cmd_out("NULL 'nsl_graph_list'");

        return (nsl_data*) 0;
}
nsl_data* nsl_system::get_graph(int index)
{
        if (nsl_graph_list != NULL)
        {
           data_type_iterator next(*nsl_graph_list);
           nsl_data* graph;
           while (graph = next())
           {
		if (index == graph->get_index())
			return graph;
	   }
        }
        else
           cmd_out("NULL 'nsl_graph_list'");

        return (nsl_data*) 0;
}
int nsl_system::init_graph_list()
{
	set_graph("area_level_graph");
	set_graph("spatial_graph_2");
	set_graph("spatial_graph_3");
	set_graph("temporal_graph");
	set_graph("user_graph");

	return graph_total;
}
void nsl_system::print_graph_list()
{
        if (nsl_graph_list != NULL)
        {
           data_type_iterator next(*nsl_graph_list);
           nsl_data* graph;
           while (graph = next())
                graph->print_status(cout);
        }
        else
           cmd_out("NULL 'nsl_graph_list'");
}
