//
//	nsl_vector_conv_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

// zero-edge vector convolution

nsl_vector NSLconv_zero(nsl_vector& a,nsl_vector& b)
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();

	int simax = saimax + sbimax - 1;

	nsl_vector c(sbimax);
	nsl_vector d(simax);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

	d = d.put_sector(b,saimax/2); 

	for (int i = 0; i < sbimax; i++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
			val = val + av[m] * dv[i+m];
		cv[i] = val;
	   }
	return c;
}
nsl_vector NSLconv_zero(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * b;

	return c;
}
nsl_vector NSLconv_zero(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = a * bv[i];

	return c;
}
nsl_vector NSLconv_zero(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * bv;

	return c;
}
nsl_vector NSLconv_zero(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type av = a.get_data();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = av * bv[i];

	return c;
}

// wrap around convolution

nsl_vector NSLconv_wrap(nsl_vector& a,nsl_vector& b)
{
      	int saimax = a.get_imax();
      	int sbimax = b.get_imax();

	int sm = saimax/2;

      	int simax = saimax + sbimax - 1;

      	nsl_vector c(sbimax);
      	nsl_vector d(simax);

      	d = d.put_sector(b,saimax/2); 

	int i,m;

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

      	for (i = sm-1; i >= 0; i--)
	   dv[i] = bv[(i-sm+sbimax)%sbimax];
      	for (i = sm+sbimax; i < simax; i++)
	   dv[i] = bv[(i-sm)%sbimax];

     	for (i = 0; i < sbimax; i++)
        {    
              num_type val = (num_type) 0;
              for (m = 0; m < saimax; m++)
                      val = val + av[m] * dv[i+m];
              cv[i] = val;
        }

        return c;
}

// copy boundary convolution

nsl_vector NSLconv_copy(nsl_vector& a,nsl_vector& b)
{
      	int saimax = a.get_imax();
      	int sbimax = b.get_imax();

	int sm = saimax/2;

      	int simax = saimax + sbimax - 1;

      	nsl_vector c(sbimax);
      	nsl_vector d(simax);

      	d = d.put_sector(b,saimax/2); 

	int i,m;

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

      	for (i = sm-1; i >= 0; i--)
	   dv[i] = bv[0];
      	for (i = sm+sbimax; i < simax; i++)
	   dv[i] = bv[sbimax-1];

      	for (i = 0; i < sbimax; i++)
        {    
              num_type val = (num_type) 0;
              for (m = 0; m < saimax; m++)
                      val = val + av[m] * dv[i+m];
              cv[i] = val;
        }

        return c;
}

