//
//	nsl_vector_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// vector threshold functions

// NSLstep

// if a > 0 then 1 else 0

nsl_vector NSLstep(nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > (num_type) 0)
			c.v[i] = (num_type) 1;
		else
			c.v[i] = 0;
	return c;
}

// if a > b then 1 else 0

nsl_vector NSLstep(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = (num_type) 1;
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b)
			c.v[i] = (num_type) 1;
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v)
			c.v[i] = (num_type) 1;
		else
			c.v[i] = 0;
	return c;
}

// if a > b then y2 else y1

nsl_vector NSLstep(nsl_vector& a,nsl_vector& b,nsl_vector& y1,nsl_vector& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = y2.v[i];
		else
			c.v[i] = y1.v[i];
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_vector& b,num_type y1,num_type y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = y2;
		else
			c.v[i] = y1;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_vector& b,nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = y2.v;
		else
			c.v[i] = y1.v;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,num_type b,num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b)
			c.v[i] = y2;
		else
			c.v[i] = y1;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v)
			c.v[i] = y2.v;
		else
			c.v[i] = y1.v;
	return c;
}

// NSLramp

// if a > 0 then a else 0

nsl_vector NSLramp(nsl_vector& a) 
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > (num_type) 0)
			c.v[i] = a.v[i];
		else
			c.v[i] = 0;
	return c;
}

// if a > b then a - b else 0

nsl_vector NSLramp(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = a.v[i] - b.v[i];
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b)
			c.v[i] = a.v[i] - b;
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v)
			c.v[i] = a.v[i] - b.v;
		else
			c.v[i] = 0;
	return c;
}

// if a > b then a - b + y2 else y1

nsl_vector NSLramp(nsl_vector& a,nsl_vector& b,nsl_vector& y1,nsl_vector& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = a.v[i] - b.v[i] + y2.v[i];
		else
			c.v[i] = y1.v[i];
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_vector& b,num_type y1,num_type y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = a.v[i] - b.v[i] + y2;
		else
			c.v[i] = y1;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_vector& b,nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = a.v[i] - b.v[i] + y2.v;
		else
			c.v[i] = y1.v;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,num_type b,num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b)
			c.v[i] = a.v[i] - b + y2;
		else
			c.v[i] = y1;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v)
			c.v[i] = a.v[i] - b.v + y2.v;
		else
			c.v[i] = y1.v;
	return c;
}

// NSLsat

nsl_vector NSLsat(nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > (num_type) 1)
			c.v[i] = (num_type) 1;
		else if (a.v[i] > 0)
			c.v[i] = a.v[i];
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2)
{
	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2.v[i])
			c.v[i] = (num_type) 1;
		else if (a.v[i] > x1.v[i])
			c.v[i] = (a.v[i]-x1.v[i])/(x2.v[i]-x1.v[i]);
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,num_type x1,num_type x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2)
			c.v[i] = (num_type) 1;
		else if (a.v[i] > x1)
			c.v[i] = (a.v[i] - x1)/(x2 - x1);
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_data& x1,nsl_data& x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2.v)
			c.v[i] = (num_type) 1;
		else if (a.v[i] > x1.v)
			c.v[i] = (a.v[i] - x1.v)/(x2.v - x1.v);
		else
			c.v[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_vector& y1,nsl_vector& y2)
{
	if (a.get_imax() != x1.get_imax() ||  a.get_imax() != x2.get_imax() || 
	    a.get_imax() != y1.get_imax() ||  a.get_imax() != y2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("imax4: ",y1.get_imax());
		cmd_error("imax5: ",y2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2.v[i])
		    c.v[i] = y2.v[i];
		else if (a.v[i] > x1.v[i])
		    c.v[i] = (y2.v[i]-y1.v[i])*(a.v[i]-x1.v[i])/
				(x2.v[i]-x1.v[i]) + y1.v[i];
		else
		    c.v[i] = y1.v[i];
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	num_type y1,num_type y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2.v[i])
		    c.v[i] = y2;
		else if (a.v[i] > x1.v[i])
		    c.v[i] = (y2-y1)*(a.v[i]-x1.v[i])/
				(x2.v[i]-x1.v[i]) + y1;
		else
		    c.v[i] = y1;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2.v[i])
		    c.v[i] = y2.v;
		else if (a.v[i] > x1.v[i])
		    c.v[i] = (y2.v-y1.v)*(a.v[i]-x1.v[i])/
				(x2.v[i]-x1.v[i]) + y1.v;
		else
		    c.v[i] = y1.v;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2)
		    c.v[i] = y2;
		else if (a.v[i] > x1)
		    c.v[i] = (y2-y1)*(a.v[i]-x1)/(x2-x1) + y1;
		else
		    c.v[i] = y1;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > x2.v)
		    c.v[i] = y2.v;
		else if (a.v[i] > x1.v)
		    c.v[i] = (y2.v-y1.v)*(a.v[i]-x1.v)/(x2.v-x1.v) + y1.v;
		else
		    c.v[i] = y1.v;
	return c;
}

// NSLsigma

nsl_vector NSLsigma(nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
	   if (a.v[i] > 1.0)
		c.v[i] = 1.0;
	   else if (a.v[i] > 0)
		c.v[i] = pow(a.v[i],2.0)*(3.0-2.0*a.v[i]);
	   else
		c.v[i] = 0;
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2)
{
 	int s1 = a.get_imax();
	nsl_vector c(s1);
	num_type sigma;
	for (int i = 0; i < s1; i++)
	{
            sigma = (a.v[i]-x1.v[i])/(x2.v[i]-x1.v[i]);
	    if (a.v[i] > x2.v[i])
		    c.v[i] = 1.0;
	    else if (a.v[i] > x1.v[i])
		    c.v[i] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    else
		    c.v[i] = 0;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,num_type x1,num_type x2)
{
 	int s1 = a.get_imax();
	nsl_vector c(s1);
	num_type sigma;
	for (int i = 0; i < s1; i++)
	{
            sigma = (a.v[i]-x1)/(x2-x1);
	    if (a.v[i] > x2)
		    c.v[i] = 1.0;
	    else if (a.v[i] > x1)
		    c.v[i] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    else
		    c.v[i] = 0;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_data& x1,nsl_data& x2)
{
 	int s1 = a.get_imax();
	nsl_vector c(s1);
	num_type sigma;
	for (int i = 0; i < s1; i++)
	{
            sigma = (a.v[i]-x1.v)/(x2.v-x1.v);
	    if (a.v[i] > x2.v)
		    c.v[i] = 1.0;
	    else if (a.v[i] > x1.v)
		    c.v[i] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    else
		    c.v[i] = 0;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_vector& y1,nsl_vector& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (a.v[i]-x1.v[i])/(x2.v[i]-x1.v[i]);
	    if (a.v[i] > x2.v[i])
		    c.v[i] = y2.v[i];
	    else if (a.v[i] > x1.v[i])
		    c.v[i] = (y2.v[i]-y1.v[i])*pow(sigma,2.0)*(3.0-2.0*sigma)
			 + y1.v[i];
	    else
		    c.v[i] = y1.v[i];
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (a.v[i]-x1.v[i])/(x2.v[i]-x1.v[i]);
	    if (a.v[i] > x2.v[i])
		    c.v[i] = y2;
	    else if (a.v[i] > x1.v[i])
		    c.v[i] = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	    else
		    c.v[i] = y1;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (a.v[i]-x1.v[i])/(x2.v[i]-x1.v[i]);
	    if (a.v[i] > x2.v[i])
		    c.v[i] = y2.v;
	    else if (a.v[i] > x1.v[i])
		    c.v[i] = (y2.v-y1.v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1.v;
	    else
		    c.v[i] = y1.v;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (a.v[i]-x1)/(x2-x1);
	    if (a.v[i] > x2)
		    c.v[i] = y2;
	    else if (a.v[i] > x1)
		    c.v[i] = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	    else
		    c.v[i] = y1;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (a.v[i]-x1.v)/(x2.v-x1.v);
	    if (a.v[i] > x2.v)
		    c.v[i] = y2.v;
	    else if (a.v[i] > x1.v)
		    c.v[i] = (y2.v-y1.v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1.v;
	    else
		    c.v[i] = y1.v;
	}
	return c;
}

// NSLmax

num_type NSLmax(nsl_vector& a) 
{
	int s1 = a.get_imax();

	num_type c = 0;

	for (int i = 0; i < s1; i++)
		if (a.v[i] > c)
			c = a.v[i];

	return c;
}

// if a > b then a else b

nsl_vector NSLmax(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
		cmd_error_exit("bad vector sector");
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v[i])
			c.v[i] = a.v[i];
		else
			c.v[i] = b.v[i];
	return c;
}
nsl_vector NSLmax(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b.v)
			c.v[i] = a.v[i];
		else
			c.v[i] = b.v;
	return c;
}
nsl_vector NSLmax(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] > b)
			c.v[i] = a.v[i];
		else
			c.v[i] = b;
	return c;
}

// NSLmin

num_type NSLmin(nsl_vector& a) 
{
	int s1 = a.get_imax();

	num_type c = 0;

	for (int i = 0; i < s1; i++)
		if (a.v[i] < c)
			c = a.v[i];

	return c;
}

// if a < b then a else b

nsl_vector NSLmin(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
		cmd_error_exit("bad vector sector");
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] < b.v[i])
			c.v[i] = a.v[i];
		else
			c.v[i] = b.v[i];
	return c;
}
nsl_vector NSLmin(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] < b.v)
			c.v[i] = a.v[i];
		else
			c.v[i] = b.v;
	return c;
}
nsl_vector NSLmin(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);
	for (int i = 0; i < s1; i++)
		if (a.v[i] < b)
			c.v[i] = a.v[i];
		else
			c.v[i] = b;
	return c;
}

// gaussian 

nsl_vector NSLgauss(nsl_data& sig,int xn)
{
	nsl_vector c(xn);
	float m = xn/2;

	for (int i=0; i < xn; i++)
	    c.elem(i) = (float) exp(-(i-m)*(i-m)/(2*sig.elem()*sig.elem()));

	return c;
}

// NSLeuler method

nsl_vector NSLeuler(nsl_vector& V,nsl_vector& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,nsl_vector& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,nsl_data& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,nsl_data& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,num_type S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,num_type S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
