/**************************************************************/
/*                                                            */
/*                       nsl_command.c                        */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

# include "nsl_command.h"

extern "C" int fork();
extern "C" int chdir(char*);

extern "C" void notify_do_winds();
extern "C" void notify_winds();

nsl_system* NSL_SYSTEM_OBJ;

extern int BREAKFG;  /* 95/6/22 aa */

nsl_file* NSL_FILE;
fstream* LOG_FILE;	// file pointer
int LOG_FILE_FG; 	// 1 - log is on
int LOG_CMD_FG;		// 1 - log command

fstream LOG_STREAM;
nsl_buffer* NSL_BUFFER;

int VERBOSE_FG = 1;
int DEBUG_FG = 0;   
int SIM_FG = 0;
int SIM_TIME_FG = 0;

int (*NSL_USER_CMD)(istream&); // for user interpreter commands

/* b 95/2/8 ama */
long dump_yes=1;  /* global */
static long shells_yes=1;
static long inputfile_yes=0;
static char infilename[MAX_LN_FILENAME];
/* e 95/2/8 ama */

NSL_DIFF DIFF; // extern 

extern void init_notifier();
extern void start_notifier();

extern void init_nsl_interrupt_handler();
extern void init_frame_extern(int*,char**);

extern int load_batch_file(char*);

extern void nsl_help_command(istream&);
extern void nsl_help_status();

extern int nsl_enable_command(istream&);
extern int nsl_disable_command(istream&);
extern int nsl_status_command(istream&);
extern int nsl_update_command(istream&);
extern int nsl_create_command(istream&);
extern int nsl_set_command(istream&);
extern int nsl_reset_command(istream&);
extern int nsl_load_command(istream&);
extern int nsl_write_command(istream&);
extern int nsl_open_command(istream&);
extern int nsl_win_dump_command(istream&);
extern int nsl_print_dump_command(istream&);
extern int nsl_file_enable_command(istream&);
extern int nsl_file_disable_command(istream&);
extern int nsl_close_command(istream&);
extern int nsl_run_command(istream&);
extern int nsl_cont_command(istream&);
extern int nsl_init_command(istream&);
extern int nsl_step_command(istream&);
extern int nsl_exec_command(istream&);

void set_global_model_pointers(); 
void set_global_display_pointers();

// putback (insert) chars into input stream for MACROS 

// ******************* nsl commands ********************** //

int nsl_skip_comment(istream& in_file)
{
	string str;
	char c;
	int status = 0;

	strcpy(str,"");
	c = 0;

	if (in_file && in_file.peek() == EOF)
		return 0;

	while (in_file && in_file.peek() == ' ' || in_file.peek() == '\t' ||
		in_file.peek() == '\n')
	{
		in_file.get(c);
	}
 
	if (in_file.peek() == '/' && in_file.get(c) && in_file.peek() == '/')
	{
		while (in_file && in_file.get(c) && c != '\n')
			;
	}

	return status;
}
int nsl_get_line(istream& in_file,char* str)
{
	char c = 0;
	int status = 0;

	strcpy(str,"");

	if (in_file && in_file.peek() == EOF) // needed ?
		return 0;

	if (in_file)
	{
		status = 1;
		in_file.get(str,80,'\n');
		in_file.get(c);  // read endl
	}
	
	if (NSL_BUFFER->get_able_fg() == 1)
		(*NSL_BUFFER) << str; // check how many words here

	return status;
}
int nsl_get_str(istream& in_file,char* ret_str)
{
	string str;
	char c=0,d=0;
	int status=0;
	int type=0; // 0-reg. 1-comments('//')

	strcpy(ret_str,""); 
	strcpy(str,"");

	if (in_file && in_file.peek() == EOF)
	{
		strcpy(ret_str,"EOF");
		return 0;
	}

// get rid of spaces
	while (in_file && in_file.get(c) && (c == ' ' || c == '\t'))
		;

	in_file.putback(c); // return last char

	// read into str buffer, w/o spaces and endlines
	int i = 0;
	while (in_file && in_file.get(c) && c != '\n' && c != ' ' && c != '\t')
		str[i++] = c;
	int len = i;

	ret_str[0] = '\0';
	if (len == 0) // empty string
	{
		status = 0;
	}
	else // string
	{
	    status = 1;
	    i = 1;
	    ret_str[0] = str[0];
	    while (i < len)
	    {
		if (str[i-1] == '/' && str[i] == '/') // get rid of all comments
		{
		    len = i-1;
	   	    ret_str[len] = '\0';
		    while (in_file && c != '\n')
			in_file.get(c);
		    if (len == 0)
			status = 0;
		}
		else
		    ret_str[i] = str[i];
		i++;
	    }
	    if (status == 1)
	    {
	        ret_str[len] = '\0';

	    	if (NSL_BUFFER->get_able_fg() == 1)
		    (*NSL_BUFFER) << ret_str;
	    }
	}

	if (c == '\n') // read endl
	{
	    if (status == 1) // len > 0
		in_file.putback(c);
	}

	return status;
}
void nsl_log_command()
{
	if (NSL_BUFFER->get_able_fg() == 1)
	{
	    (*NSL_BUFFER) >> cout;
	    if (LOG_FILE_FG == 1 && LOG_FILE != NULL && LOG_CMD_FG == 1)
	    {
	    	NSL_BUFFER->rewind_stream();
		(*NSL_BUFFER) >> (*LOG_FILE);
	    }
	}
}
int nsl_shell(istream& in_file)
{
	string str1,str2;
	int i,i0;

	int status = 1;

//	set_global_pointers(); 		// set model
//	set_global_model_pointers(); 	// set module and layer ptrs
//	set_global_display_pointers();

	if (NSL_BUFFER->get_able_fg() == 1)
	    NSL_BUFFER->reset_stream();

	if (nsl_get_str(in_file,str1) == 0)
		return 1;

	LOG_CMD_FG = 1;

// nsl commands
	if (strcmp(str1,"help") == 0)
	{
		nsl_help_command(in_file);
		LOG_CMD_FG = 0;
	}
	else if (strcmp(str1,"create") == 0)
	{
		status = nsl_create_command(in_file);
	}
	else if (strcmp(str1,"disable") == 0)
	{
		status = nsl_disable_command(in_file);
	}
	else if (strcmp(str1,"enable") == 0)
	{
		status = nsl_enable_command(in_file);
	}
	else if (strcmp(str1,"exec") == 0)
	{
		status = nsl_exec_command(in_file);
	}
	else if (strcmp(str1,"exit") == 0 || strcmp(str1,"quit") == 0)
	{
		LOG_CMD_FG = 0;
		status = 0;
		notify_stop();
	}
// file commands
	else if (strcmp(str1,"file_enable") == 0)
	{
		status = nsl_file_enable_command(in_file);
	}
	else if (strcmp(str1,"file_disable") == 0)
	{
		status = nsl_file_disable_command(in_file);
	}
	else if (strcmp(str1,"close") == 0)
	{
		status = nsl_close_command(in_file);
	}
	else if (strcmp(str1,"load") == 0 || strcmp(str1,"read") == 0)
	{
//		LOG_CMD_FG = 0;
		status = nsl_load_command(in_file);
	}
	else if (strcmp(str1,"open") == 0)
	{
		status = nsl_open_command(in_file);
	}
	else if (strcmp(str1,"write") == 0)
	{
		status = nsl_write_command(in_file);
	}
// print commands
	else if (strcmp(str1,"print_dump") == 0)
	{
	  if (!dump_yes) {
	       printf("\nSorry, print commands not available in WEB NSL.\n\n");
	       printf("You must be logged in directly to invoke this command.\n\n");
	  } else {
		status = nsl_print_dump_command(in_file);
	      }
	}
	else if (strcmp(str1,"win_dump") == 0)
	{
	  if (!dump_yes) {
	       printf("\nSorry, print commands not available in WEB NSL.\n\n");
	       printf("You must be logged in directly to invoke this command.\n\n");
	  } else {
		status = nsl_win_dump_command(in_file);
	      }
	}
// proc commands
	else if (strcmp(str1,"cont") == 0)
	{
		status = nsl_cont_command(in_file);
	}
	else if (strcmp(str1,"init") == 0)
	{
		status = nsl_init_command(in_file);
	}
	else if (strcmp(str1,"run") == 0)
	{
		status = nsl_run_command(in_file);
	}
	else if (strcmp(str1,"step") == 0)
	{
		status = nsl_step_command(in_file);
	}
	else if (strcmp(str1,"reset") == 0)
	{
		status = nsl_reset_command(in_file);
	}
	else if (strcmp(str1,"set") == 0)
	{
		status = nsl_set_command(in_file);
	}
// shell commands
	else if (strcmp(str1,"cd") == 0)
	{
	   if (nsl_get_str(in_file,str2) != 0) {
	     if (!shells_yes) {
	       printf("\nSorry, shell commands not available in WWW NSL.\n\n");
	       printf("You must be logged in directly to invoke this command.\n\n");
	     }
	     else {
	       chdir(str2);
	     }
	   }
	 }
//	else if (strcmp(str1,"pwd") == 0)
//	{
//	}
	else if (strcmp(str1,"csh") == 0)
	{
	  if (!shells_yes) {
	    printf("\nSorry, shell commands not available in WEB NSL.\n\n");
	    printf("You must be logged in directly to invoke this command.\n\n");
	  } else {
	   if (nsl_get_line(in_file,str2) != 0) {
	      	if (fork() == 0) {
			execl("/bin/csh","csh","-c",str2,(char*) 0);
	                perror("execl");
		}
	      }
	 }
	}
	else if (strcmp(str1,"sh") == 0)
	{
	  if (!shells_yes) {
  	    printf("\nSorry, shell commands not available in WEB NSL.\n\n");
	    printf("You must be logged in directly to invoke this command.\n\n");
	  } else {
	   if (nsl_get_line(in_file,str2) != 0) {
		system(str2);
	      }
	 }
	}
	else if (strcmp(str1,"status") == 0 || strcmp(str1,"print") == 0)
	{
		status = nsl_status_command(in_file);
	}
	else if (strcmp(str1,"update") == 0)
	{
		status = nsl_update_command(in_file);
	}
	else if (strcmp(str1,"user") == 0)
	{
	   if (NSL_USER_CMD != NULL)
		status = NSL_USER_CMD(in_file);
	   else
		cmd_error("NULL 'NSL_USER_CMD' file pointer");
	}
	else
	{
		LOG_CMD_FG = 0;
	        cmd_error("Unknown Command: ",str1);
	}

	nsl_log_command();

	return status;
}
void NSLinit(int argc,char** argv)
{
        extern char *optarg;
        extern int optind;

	char c = 'a';
	long errflg=0;
	long i=0;
	long infileln=0;

	/* 95/6/22 aa initialize BREAKFG in graph_winds.c*/
	BREAKFG=0;

	if (NSL_SYSTEM_OBJ == NULL)
		NSL_SYSTEM_OBJ = new nsl_system;

	NSL_SYSTEM = NSL_SYSTEM_OBJ;

	CMD_BUF = new command_buffer;



	NSL_BUFFER = new nsl_buffer();

	cin.sync_with_stdio(); // sync stdio with C++ streams
	cout.sync_with_stdio(); // sync stdio with C++ streams
	cerr.sync_with_stdio(); // sync stdio with C++ streams
	LOG_STREAM.attach(1); // stdout (currently used only for status cmds)

	init_nsl_interrupt_handler();

	cmd_out("NSL Version 2.1.3 - 6/22/95");
	cmd_out("Initializing NSL Simulation System");


	/* read parameter arguments */

 	/* b ama 95/5/15 */
	/* man 3 getopt */
	while ((c = getopt(argc, argv, "sdi:")) != -1) {
          switch (c) {
            case 's':
	      shells_yes=0;
              break;
            case 'd':
	      dump_yes=0;
              break;
            case 'i':
	      inputfile_yes=1;
	      strncpy(infilename,optarg,(MAX_LN_FILENAME-1));
	      infilename[MAX_LN_FILENAME-1]='\0';
/* 95/6/22 aa not needed
	      if ((strstr(infilename,".nsl")!=NULL)||
		  (strstr(infilename,".NSL")!=NULL)) {
		infileln=strlen(infilename);
		infilename[infileln-4]='\0';		
	      }
*/
              break;
            case '?':
              errflg++;
	      break;
            default:
              errflg++;
	      break;
	    }
          if (errflg) {
             (void)fprintf(stderr, "Error in usage.\n");
             (void)fprintf(stderr, "Usage: nsl -s -i:your_name_here.nsl -- <whatever else you wanted as parameters>.\n\n ");
             (void)fprintf(stderr, "s - means user NOT allowed to have shells.\n");
             (void)fprintf(stderr, "d - means user NOT allowed to perform window dumps.\n");
             (void)fprintf(stderr, "i - means input file.\n");
              exit (0);
	   }
	}  /* end while */
	/* now send everything after -- to NSL */
        if (optind< argc) {
          for (i=1; (optind<argc); optind++,i++) {
                  argv[i]=argv[optind];
          }
          argc= argc - optind + 1; /* one for argv0 */
        }

	NSL_SYSTEM->set_argc_ptr(&argc);
	NSL_SYSTEM->set_argv_ptr(argv);

 	/* e ama 95/5/15 */ 
}

void NSLmain(int argc,char** argv)
{
/*	NSL_FILE = new nsl_file();
	NSL_FILE->set_type(1); // log file
	NSL_SYSTEM->add_file(NSL_FILE);
	NSL_FILE->open(1);
	LOG_FILE = NSL_FILE->get_file_stream();
	LOG_FILE_FG = 1;
*/
        nsl_interpreter NSL2;

	set_global_pointers(); 		// set model

	cout << "nsl> ";
	cout.flush();

	init_notifier();

	/* b aa 95/5/16 */
	if (inputfile_yes) {
	  NSL2<< "load " << infilename << "\n";
	  cout << "nsl> ";
  	  cout.flush();
	}
	/* e aa 95/5/16 */

	start_notifier();

	cmd_out("Exiting NSL Simulation System");

//	NSL_FILE->close();
//	NSL_SYSTEM->print_all_file_status(cout);
}

